"""
PortiaPy
========

A Python stub for Agriness Edge's Portia REST API. 

__authors__ = ['Lucas Góes', 'Matheus Mota']
__emails__ = ['lucas@agrinessedge.com', 'matheus@agrinessedge.com']
"""

from .version import __version__


__all__ = ['axioms', 'describe', 'phases', 'portia', 'profile', 'select',
		   'specs', 'summary', 'utils']


name = 'portiapy'

portiaConfigExample = {
	'baseurl': 'http://localhost',
	'authorization': '<your API key>',
	'debug': False,
	'Accept': 'text/csv'
}

NAME = name

PORTIA_CONFIG_EXAMPLE = portiaConfigExample


def about():
    print('portiapy - an Agriness Edge project')

def version():
	print('portiapy - v{0}'.format(__version__))



"""The VERSION file is the sole source of the project's current version. This
script was created exclusively to fetch its content for __main__.py usage.
"""

from os.path import dirname, join


def version() -> str:
    """Returns package version parsed from the VERSION file."""
    with open(join(dirname(__file__), 'VERSION')) as f:
        return f.read().strip()


__version__ = version()



#####################################
#               Axioms              #
#####################################

# Libraries
import json                       # JSON encoder and decoder
import portiapy.utils as utils

# Functions
def about():
    print("portiapy.axioms - an Agriness Edge project")

########################################
# index
########################################
# /pipeline/axioms
########################################
def index(portiaConfig):

	endpoint = '/pipeline/axioms'
	response = utils.httpGetRequest(portiaConfig, endpoint)

	if response.status_code == 200:

		d = json.loads(response.text)
		if portiaConfig['debug']:
			print( '[portia-debug]: {0}'.format(d) )

		return d

	else:

		d = json.loads(response.text)
		raise Exception( 'couldn\'t list axioms: {0}'.format(d['message']) )

########################################
# store
########################################
# /pipeline/axioms
########################################
def store(portiaConfig, payload):

	endpoint = '/pipeline/axioms'
	response = utils.httpPostRequest(portiaConfig, endpoint, payload)

	if response.status_code == 200:

		d = json.loads(response.text)
		if portiaConfig['debug']:
			print( '[portia-debug]: {0}'.format(d) )

		return d

	else:

		d = json.loads(response.text)
		raise Exception( 'couldn\'t create axiom: {0}'.format(d['message']) )

########################################
# show
########################################
# /pipeline/axioms/:axiom
########################################
def show(portiaConfig, axiomName, params=False):

	if params == False:
		endpoint = '/pipeline/axioms/{0}'.format(axiomName)
	else:
		endpoint = '/pipeline/axioms/{0}/params'.format(axiomName)

	response = utils.httpGetRequest(portiaConfig, endpoint)

	if response.status_code == 200:

		d = json.loads(response.text)
		if portiaConfig['debug']:
			print( '[portia-debug]: {0}'.format(d) )

		return d

	else:

		d = json.loads(response.text)
		raise Exception( 'couldn\'t show axiom: {0}'.format(d['message']) )

########################################
# update
########################################
# /pipeline/axioms/:axiom
########################################
def update(portiaConfig, axiomName, payload):

	endpoint = '/pipeline/axioms/{0}'.format(axiomName)
	response = utils.httpPutRequest(portiaConfig, endpoint, payload)

	if response.status_code == 200:

		d = json.loads(response.text)
		if portiaConfig['debug']:
			print( '[portia-debug]: {0}'.format(d) )

		return d

	else:

		d = json.loads(response.text)
		raise Exception( 'couldn\'t update axiom: {0}'.format(d['message']) )

########################################
# destroy
########################################
# /pipeline/axioms/:axiom
########################################
def destroy(portiaConfig, axiomName):

	endpoint = '/pipeline/axioms/{0}'.format(axiomName)
	response = utils.httpDeleteRequest(portiaConfig, endpoint)

	if response.status_code == 200:

		d = json.loads(response.text)
		if portiaConfig['debug']:
			print( '[portia-debug]: {0}'.format(d) )

		return d

	else:

		d = json.loads(response.text)
		raise Exception( 'couldn\'t destroy axiom: {0}'.format(d['message']) )



"""Profile tools where it is possible to create snapshots of a device's state
on any given time.
"""

import json
from enum import Enum

import portiapy.utils as utils


class ProfileStrategies(Enum):
    BY_ZERO_PORT = 1
    BY_PORTS = 2

    @property
    def endpoint(self):
        return self.name.lower().replace('_', '')


def device_profile(
    portia_config: dict,
    edge_id: str,
    strategy: ProfileStrategies=ProfileStrategies.BY_ZERO_PORT,
    interval: int=30,
    params: dict={ 'sort': True, 'precision': 'ms', 'timezone': 'Etc/UTC' }
) -> dict:
    """Retrieves a device's profile.

    Arguments:
        portia_config {dict} -- Portia's configuration arguments 
        edge_id {str} -- Edge ID that identifies the device
    
    Keyword Arguments:
        strategy {ProfileStrategies} -- strategy to use when building the
                                        profile (default: 
                                        {ProfileStrategies.BY_ZERO_PORT})
        interval {int} -- interval of time in minutes to build the profile
                          (default: {30})
        params {dict} -- params to send to the service (default: 
                         {{ 'sort', 'precision': 'ms',
                         'timezone': 'Etc/UTC' }})

    Returns:
        dict -- dictionary with the device's profile
    """
    endpoint = '/profile/device/{0}/{1}/{2}'.format(
        edge_id, strategy.endpoint, interval
    )

    response = utils.http_get_request(portia_config, endpoint, params)

    if response.status_code == 200:

        d = json.loads(response.text)
        if portia_config['debug']:
            print('[portia-debug]: {0}'.format(d.get('ports')))

        return d

    else:
        raise Exception("couldn't retrieve data")


def port_profile(
    portia_config: dict,
    edge_id: str,
    port: int,
    strategy: ProfileStrategies=ProfileStrategies.BY_ZERO_PORT,
    interval: int=30,
    params: dict={ 'sort': True, 'precision': 'ms', 'timezone': 'Etc/UTC' }
) -> dict:
    """Retrieves a port's profile.

    Arguments:
        portia_config {dict} -- Portia's configuration arguments
        edge_id {str} -- Edge ID that identifies the device
        port {int} -- port of the device
    
    Keyword Arguments:
        strategy {ProfileStrategies} -- strategy to use when building the
                                        profile (default: 
                                        {ProfileStrategies.BY_ZERO_PORT})
        interval {int} -- interval of time in minutes to build the profile
                          (default: {30})
        params {dict} -- params to send to the service (default: 
                         {{ 'sort', 'precision': 'ms',
                         'timezone': 'Etc/UTC' }})

    Returns:
        dict -- dictionary with the port's profile
    """
    endpoint = '/profile/device/{0}/port/{1}/{2}/{3}'.format(
        edge_id, port, strategy.endpoint, interval
    )

    response = utils.http_get_request(portia_config, endpoint, params)

    if response.status_code == 200:

        d = json.loads(response.text)
        if portia_config['debug']:
            print('[portia-debug]: {0}'.format(d.get('ports')))

        return d

    else:
        raise Exception("couldn't retrieve data")


def sensor_profile(
    portia_config: dict,
    edge_id: str,
    port: int,
    sensor: int,
    strategy: ProfileStrategies=ProfileStrategies.BY_ZERO_PORT,
    interval: int=30,
    params: dict={ 'sort': True, 'precision': 'ms', 'timezone': 'Etc/UTC' }
) -> dict:
    """Retrieves a sensor's profile.

    Arguments:
        portia_config {dict} -- Portia's configuration arguments
        edge_id {str} -- Edge ID that identifies the device
        port {int} -- port of the device
        sensor {int} -- sensor of the device
    
    Keyword Arguments:
        strategy {ProfileStrategies} -- strategy to use when building the
                                        profile (default: 
                                        {ProfileStrategies.BY_ZERO_PORT})
        interval {int} -- interval of time in minutes to build the profile
                          (default: {30})
        params {dict} -- params to send to the service (default: 
                         {{ 'sort', 'precision': 'ms',
                         'timezone': 'Etc/UTC' }})

    Returns:
        dict -- dictionary with the sensor's profile
    """
    endpoint = '/profile/device/{0}/port/{1}/sensor/{2}/{3}/{4}'.format(
        edge_id, port, sensor, strategy.endpoint, interval
    )

    response = utils.http_get_request(portia_config, endpoint, params)

    if response.status_code == 200:

        d = json.loads(response.text)
        if portia_config['debug']:
            print('[portia-debug]: {0}'.format(d.get('ports')))

        return d

    else:
        raise Exception("couldn't retrieve data")



"""Utility constants, methods and classes used by all modules.
"""

import json
import time
from io import StringIO

import arrow
import requests
import pandas as pd
from dateutil import tz
import plotly.offline as plotly
import plotly.graph_objs as plotlygo


THING_CODES = {
    0:  'NotSpecified',
    1:  'Sensor_Agriness_TU',
    2:  'Sensor_Agriness_AirQ',
    3:  'Collector_Agriness_Hydro_v1',
    4:  'Sensor_Agriness_AnimalScale',
    5:  'Collector_Agriness_CycleCounter_v1',
    6:  'Collector_Inobram_Smaai_v5',
    7:  'Sensor_Agriness_HydroEDN15-100',
    8:  'Sensor_Agriness_HydroQMS',
    9:  'Sensor_Agriness_HydroTemp',
    10: 'T4_Agriness_v0',
    11: 'Collector_Agriness_Universal_v1',
    12: 'Collector_Inobram_Smaai_v3',
    13: 'Collector_Inobram_Smaai_v4',
    14: 'T4_Agriness_v1',
    15: 'Sensor_Inobram_T',
    16: 'Sensor_Inobram_TU',
    17: 'Sensor_Inobram_Exhaustor',
    18: 'Sensor_Inobram_Nebulizer',
    19: 'Sensor_Inobram_Heater',
    20: 'Sensor_Inobram_Dimmer',
    21: 'Sensor_Inobram_PE',
    22: 'Sensor_Inobram_H2O',
    23: 'Sensor_Inobram_CO2',
    24: 'Sensor_Inobram_U',
    25: 'Sensor_Inobram_AnimalScale',
    26: 'Sensor_Inobram_SiloScale',
    27: 'T4_Agriness_Virtual_v0',
    28: 'Collector_Inobram_Ambientte_v1',
    29: 'Collector_Agriness_Bluetooth_v1',
    30: 'Sensor_Datamars_AnimalScale_v2',
    31: 'T4_Agriness_Network_v0',
    32: 'Sensor_Inobram_Inlet',
    33: 'Sensor_Inobram_Curtain',
    34: 'Sensor_Datamars_AnimalScale_v3',
    35: 'Sensor_GSI_T',
    36: 'Sensor_GSI_U',
    37: 'Sensor_GSI_H2O',
    38: 'Sensor_GSI_AnimalScale',
    39: 'Sensor_GSI_Exhaustor',
    40: 'Collector_AgNet_Gateway_v1',
    41: 'Sensor_AgNet_T1',
    42: 'Sensor_AgNet_U',
    43: 'Collector_Roboagro_v1',
    44: 'Collector_GSI_EDGE_v1',
    45: 'Collector_BigDutchman_Viper_v1',
    46: 'Sensor_BigDutchman_T',
    47: 'Sensor_BigDutchman_U',
    48: 'Sensor_BigDutchman_H2O',
    49: 'Sensor_BigDutchman_AnimalScale',
    50: 'Sensor_BigDutchman_PE',
    51: 'Sensor_BigDutchman_NH3',
    52: 'Sensor_BigDutchman_CO2',
    53: 'T4_Agriness_v2',
    54: 'Sensor_AgNet_Light',
    55: 'Sensor_AgNet_ExtT',
    56: 'Sensor_AgNet_ExtU',
    57: 'Sensor_AgNet_H2O',
    58: 'Sensor_AgNet_CO2',
    59: 'Sensor_AgNet_NH3',
    60: 'Sensor_AgNet_FeedC',
    61: 'Sensor_AgNet_AirS',
    62: 'Sensor_AgNet_AnimalScale',
    63: 'T4_Agriness_Virtual_v1',
    64: 'Sensor_AgNet_T2',
    65: 'Sensor_AgNet_T3',
    66: 'Sensor_AgNet_T4',
    67: 'Sensor_AgNet_T5',
    68: 'Sensor_Inobram_T_H2O',
    69: 'Collector_Inobram_Ambientte_v3',
    70: 'Sensor_Roboagro_Stock',
    71: 'Sensor_Roboagro_Consumption',
    72: 'Sensor_Roboagro_Animal_Counter',
    73: 'Sensor_Roboagro_Bay_Consumption',
    74: 'Sensor_Roboagro_Bay_Animal_Counter',
    75: 'Collector_Roboagro_v2',
    76: 'Collector_IonTec_v1',
    77: 'Sensor_IonTec_T',
    78: 'Sensor_IonTec_U',
    79: 'Sensor_IonTec_CO2',
    80: 'Sensor_IonTec_NH3',
    81: 'Sensor_IonTec_Feeder',
    82: 'Sensor_IonTec_ESF',
    83: 'Sensor_IonTec_Water_Level_Meter',
    84: 'Sensor_IonTec_Environment_Meter',
    85: 'Sensor_IonTec_Feed_Bin_Meter',
    86: 'Sensor_MSD_IDAL_Twin_v3',
    87: 'Collector_Bin_Sentry_v1',
    88: 'Sensor_Bin_Sentry'
}

DIMENSION_CODES = {
    'en-us': {
        0:  'Generic Dimension',
        1:  'Point Temperature',
        2:  'Average Temperature',
        3:  'Point Humidity',
        4:  'Average Humidity',
        5:  'Point Gas Concentration',
        6:  'Average Gas Concentration',
        7:  'Accumulated Water Flow',
        8:  'Point Water Flow',
        9:  'Point Water Temperature',
        10: 'Point Weight',
        11: 'Average Weight',
        12: 'Status',
        13: 'Point CO2 Concentration',
        14: 'Counter',
        15: 'Accumulated Time',
        16: 'Signal Strength',
        17: 'Free Memory',
        18: 'Free Storage',
        19: 'System Load',
        20: 'File Size',
        21: 'Point Time',
        22: 'Profile',
        23: 'Point Electric Voltage',
        24: 'Point Pressure',
        25: 'Average Pressure',
        26: 'Model',
        27: 'Lot Day',
        28: 'Average Electric Voltage',
        29: 'Point Light',
        30: 'Point Wind Speed',
        31: 'Point NH3 Concentration',
        32: 'Point Angulation',
        33: 'Accumulated Weight',
        34: 'Point Volume'
    },
    'pt-br': {
        0:  'Dimensão Genérica',
        1:  'Temperatura Pontual',
        2:  'Temperatura Média',
        3:  'Umidade Pontual',
        4:  'Umidade Média',
        5:  'Concentração de Gases Pontual',
        6:  'Concentração de Gases Média',
        7:  'Fluxo de Água Acumulado',
        8:  'Fluxo de Água Pontual',
        9:  'Temperatura da Água Pontual',
        10: 'Peso Pontual',
        11: 'Peso Médio',
        12: 'Estado',
        13: 'Concentração de CO2 Pontual',
        14: 'Contador',
        15: 'Tempo Acumulado',
        16: 'Força de Sinal',
        17: 'Memória Livre',
        18: 'Armazenamento Livre',
        19: 'Carga do Sistema',
        20: 'Tamanho de Arquivo',
        21: 'Tempo Pontual',
        22: 'Perfil',
        23: 'Tensão Elétrica Pontual',
        24: 'Pressão Pontual',
        25: 'Pressão Média',
        26: 'Modelo',
        27: 'Dia do Lote',
        28: 'Tensão Elétrica Média',
        29: 'Luz Pontual',
        30: 'Velocidade do Vento Pontual',
        31: 'Concentração de NH3 Pontual',
        32: 'Angulação Pontual',
        33: 'Peso Acumulado',
        34: 'Volume Pontual'
    }
}

EVENT_CODES = {
    'en-us': {
        0:  'Generic Event',
        1:  'Communication State'
    },
    'pt-br': {
        0:  'Evento Genérico',
        1:  'Estado de Comunicação'
    }
}

UNITY_CODES = {
    'en-us': {
        0:  ' unit(s)',
        1:  '°C',
        2:  '%',
        3:  'ppm',
        4:  'L',
        5:  'g',
        6:  's',
        7:  'L/min',
        8:  'dBm',
        9:  'B',
        10: '',
        11: '',
        12: 'ms',
        13: 'V',
        14: 'Pa',
        15: 'kg',
        16: ' day(s)',
        17: 'mV',
        18: 'gal',
        19: '°F',
        20: 'mL',
        21: 'lx',
        22: 'm/s',
        23: 't',
        24: 'lb',
        25: 'min',
        26: '°',
        27: 'm³'
    },
    'pt-br': {
        0:  ' unidade(s)',
        1:  '°C',
        2:  '%',
        3:  'ppm',
        4:  'L',
        5:  'g',
        6:  's',
        7:  'L/min',
        8:  'dBm',
        9:  'B',
        10: '',
        11: '',
        12: 'ms',
        13: 'V',
        14: 'Pa',
        15: 'kg',
        16: ' dia(s)',
        17: 'mV',
        18: 'gal',
        19: '°F',
        20: 'mL',
        21: 'lx',
        22: 'm/s',
        23: 't',
        24: 'lb',
        25: 'min',
        26: '°',
        27: 'm³'
    }
}


def humanize_thing_code(thing_code: int) -> str:
    """Translates a thing code into a humanized version of it.
    
    Arguments:
        thing_code {int} -- thing code to be humanized
    
    Returns:
        str -- humanized thing code
    """
    translated_thing_code = THING_CODES.get(int(thing_code))

    if translated_thing_code is None:
        return 'Thing Code {}'.format(thing_code)
    else:
        return translated_thing_code

def humanize_dimension_code(
    dimension_code: int, locale: str='en-us', custom: list=None
) -> str:
    """Translates a dimension code into a humanized version of it.
    
    Arguments:
        dimension_code {int} -- dimension code to be humanized
    
    Keyword Arguments:
        locale {str} -- which language to use when humanizing
                        (default: {'en-us'})
        custom {list} -- custom list of dimension codes (default: {None})
    
    Returns:
        str -- humanized dimension code
    """
    try:
        if custom is None:
            translated_dimension_code = DIMENSION_CODES.get(locale) \
                .get(int(dimension_code))
        else:
            translated_dimension_code = custom.get(locale) \
                .get(int(dimension_code))
    except:
        return 'Dimension Code {}'.format(dimension_code)
    else:
        return translated_dimension_code

def humanize_event_code(
    event_code: int, locale: str='en-us', custom: list=None
) -> str:
    """Translates an event code into a humanized version of it.
    
    Arguments:
        event_code {int} -- event code to be humanized
    
    Keyword Arguments:
        locale {str} -- which language to use when humanizing
                        (default: {'en-us'})
        custom {list} -- custom list of event codes (default: {None})
    
    Returns:
        str -- humanized event code
    """
    try:
        if custom is None:
            translated_event_code = EVENT_CODES.get(locale) \
                .get(int(event_code))
        else:
            translated_event_code = custom.get(locale) \
                .get(int(event_code))
    except:
        return 'Event Code {}'.format(event_code)
    else:
        return translated_event_code

def humanize_unity_code(
    unity_code: int, locale: str='en-us', custom: list=None
) -> str:
    """Translates a unity code into a humanized version of it.
    
    Arguments:
        unity_code {int} -- unity code to be humanized
    
    Keyword Arguments:
        locale {str} -- which language to use when humanizing
                        (default: {'en-us'})
        custom {list} -- custom list of unity codes (default: {None})
    
    Returns:
        str -- humanized unity code
    """
    try:
        if custom is None:
            translated_unity_code = UNITY_CODES.get(locale) \
                .get(int(unity_code))
        else:
            translated_unity_code = custom.get(locale) \
                .get(int(unity_code))
    except:
        return 'Unity Code {}'.format(unity_code)
    else:
        return translated_unity_code

def humanize_dataframe(
    dataframe: 'pd.DataFrame',
    locale: str='en-us',
    custom_dimension: list=None,
    custom_event: list=None,
    custom_unity: list=None
) -> 'pd.DataFrame':
    """Humanizes a data frame's dimensions by translating its columns data to
    actual text.

    Arguments:
        dataframe {pd.DataFrame} -- data frame to be humanized
    
    Keyword Arguments:
        locale {str} -- which language to use when humanizing
                        (default {'en-us'})
        custom_dimension {dict} -- custom list of dimension codes
                                   (default {None})
        custom_event {dict} -- custom list of event codes (default {None})
        custom_unity {dict} -- custom list of unity codes (default {None})

    Returns:
        pd.DataFrame -- humanized data frame
    """
    if 'dimension_thing_code' in dataframe.columns:
        dataframe['dimension_thing'] = dataframe['dimension_thing_code'].map(
            lambda thing_code: humanize_thing_code(int(thing_code))
        )


    if 'dimension_unity_code' in dataframe.columns:
        dataframe['dimension_unity'] = dataframe['dimension_unity_code'].map(
            lambda unity_code: humanize_unity_code(
                unity_code=int(unity_code),
                locale=locale,
                custom=custom_unity
            )
        )

    if 'dimension_code' in dataframe.columns:
        dataframe['dimension'] = dataframe['dimension_code'].map(
            lambda dimension_code: humanize_dimension_code(
                dimension_code=int(dimension_code),
                locale=locale,
                custom=custom_dimension
            )
        )

    if 'event_code' in dataframe.columns:
        dataframe['event'] = dataframe['event_code'].map(
            lambda event_code: humanize_event_code(
                event_code=int(event_code),
                locale=locale,
                custom=custom_event
            )
        )

    return dataframe

def humanize_json(
    json_: dict,
    locale: str='en-us',
    custom_dimension: list=None,
    custom_unity: list=None
) -> dict:
    """Humanizes a json's dimensions by translating its columns data to actual
    text.

    Arguments:
        json_ {dict} -- json to be humanized
    
    Keyword Arguments:
        locale {str} -- which language to use when humanizing
                        (default {'en-us'})
        custom_dimension {dict} -- custom list of dimension codes
                                   (default {None})
        custom_unity {dict} -- custom list of unity codes (default {None})

    Returns:
        dict -- humanized dictionary
    """
    json_['channel'] = humanize_thing_code(json_['channel_code'])
    json_['thing'] = humanize_thing_code(json_['thing_code'])

    for port in json_.get('ports'):
        port['thing'] = humanize_thing_code(port['thing_code'])

        for sensor in port.get('sensors'):
            last_package = sensor.get('last_package')

            last_package['dimension'] = humanize_dimension_code(
                dimension_code=last_package.get('dimension_code'),
                locale=locale,
                custom=custom_dimension
            )

            last_package['dimension_unity'] = humanize_unity_code(
                unity_code=last_package.get('dimension_unity_code'),
                locale=locale,
                custom=custom_unity
            )

            last_package['dimension_thing'] = humanize_thing_code(
                last_package.get('dimension_thing_code')
            )

    return json_


def http_get_request(
    portia_config: dict,
    endpoint: str,
    params: dict=None,
    optional_headers: dict=None
) -> object:
    """Makes an HTTP GET request.
    
    Arguments:
        portia_config {dict} -- Portia's configuration arguments
        endpoint {str} -- endpoint to make the request to
    
    Keyword Arguments:
        params {dict} -- params to send to the service (default: {None})
        optional_headers {dict} -- dictionary with other headers
                                   (default: {None})

    Returns:
        object -- response object
    """
    headers = {
        'Authorization': 'Bearer {0}' \
        .format(portia_config.get('authorization'))
    }

    if optional_headers is not None:
        headers = {**headers, **optional_headers}

    start = time.time()
    response = requests.get(
        '{0}{1}'.format(portia_config.get('baseurl'), endpoint),
        headers=headers,
        params=params
    )
    end = time.time()

    if portia_config.get('debug') == True:
        print(
            '[portia-debug]: status: {0} | {1:.4f} sec. | {2}' \
            .format(response.status_code, end - start, response.url)
        )

    return response

def http_post_request(
    portia_config: dict,
    endpoint: str,
    payload: dict,
    params: dict=None,
    optional_headers: dict=None
) -> object:
    """Makes an HTTP POST request.
    
    Arguments:
        portia_config {dict} -- Portia's configuration arguments 
        endpoint {str} -- endpoint to make the request to
        payload {dict} -- payload to send to the service
    
    Keyword Arguments:
        params {dict} -- params to send to the service (default: {None})
        optional_headers {dict} -- dictionary with other headers
                                   (default: {None})

    Returns:
        object -- response object
    """
    headers = {
        'Authorization': 'Bearer {0}' \
        .format(portia_config.get('authorization'))
    }

    if optional_headers is not None:
        headers = {**headers, **optional_headers}

    start = time.time()
    response = requests.post(
        '{0}{1}'.format(portia_config.get('baseurl'), endpoint),
        headers=headers,
        params=params,
        json=payload
    )
    end = time.time()

    if portia_config.get('debug') == True:
        print(
            '[portia-debug]: status: {0} | {1:.4f} sec. | {2}' \
            .format(response.status_code, end - start, response.url)
        )

    return response

def http_put_request(
    portia_config: dict,
    endpoint: str,
    payload: dict,
    params: dict=None,
    optional_headers: dict=None
)  -> object:
    """Makes an HTTP PUT request.
    
    Arguments:
        portia_config {dict} -- Portia's configuration arguments 
        endpoint {str} -- endpoint to make the request to
        payload {dict} -- payload to send to the service
    
    Keyword Arguments:
        params {dict} -- params to send to the service (default: {None})
        optional_headers {dict} -- dictionary with other headers
                                   (default: {None})

    Returns:
        object -- response object
    """
    headers = {
        'Authorization': 'Bearer {0}' \
        .format(portia_config.get('authorization'))
    }

    if optional_headers is not None:
        headers = {**headers, **optional_headers}

    start = time.time()
    response = requests.put(
        '{0}{1}'.format(portia_config.get('baseurl'), endpoint),
        headers=headers,
        params=params,
        json=payload
    )
    end = time.time()

    if portia_config.get('debug') == True:
        print(
            '[portia-debug]: status: {0} | {1:.4f} sec. | {2}' \
            .format(response.status_code, end - start, response.url)
        )

    return response

def http_delete_request(
    portia_config: dict,
    endpoint: str,
    payload: dict=None,
    params: dict=None,
    optional_headers: dict=None
) -> object:
    """Makes an HTTP DELETE request.
    
    Arguments:
        portia_config {dict} -- Portia's configuration arguments 
        endpoint {str} -- endpoint to make the request to
    
    Keyword Arguments:
        payload {dict} -- payload to send to the service (default: {None})
        params {dict} -- params to send to the service (default: {None})
        optional_headers {dict} -- dictionary with other headers
                                   (default: {None})

    Returns:
        object -- response object
    """
    headers = {
        'Authorization': 'Bearer {0}' \
        .format(portia_config.get('authorization'))
    }

    if optional_headers is not None:
        headers = {**headers, **optional_headers}

    start = time.time()
    response = requests.delete(
        '{0}{1}'.format(portia_config.get('baseurl'), endpoint),
        headers=headers,
        params=params,
        json=payload
    )
    end = time.time()

    if portia_config.get('debug') == True:
        print(
            '[portia-debug]: status: {0} | {1:.4f} sec. | {2}' \
            .format(response.status_code, end - start, response.url.encode('utf8'))
        )

    return response


def convert_csv(portia_config: dict, response: object) -> 'pd.DataFrame':
    """Converts a CSV text file to a data frame.
    
    Arguments:
        portia_config {dict} -- Portia's configuration arguments 
        response {object} -- HTTP response object
    
    Returns:
        pd.DataFrame -- converted dataframe
    
    Raises:
        Exception -- when the conversion goes wrong
    """
    if response.status_code == 200:

        try:

            dataframe = pd.read_csv(StringIO(response.text), sep=';')

            if 'port' in dataframe.columns:
                dataframe['port'] = dataframe['port'].map(int)

            if 'sensor' in dataframe.columns:
                dataframe['sensor'] = dataframe['sensor'].map(int)

            if 'event_code' in dataframe.columns:
                dataframe['event_code'] = dataframe['event_code'].map(int)

            if 'dimension_code' in dataframe.columns:
                dataframe['dimension_code'] = dataframe['dimension_code'] \
                    .map(int)

            if 'dimension_thing_code' in dataframe.columns:
                dataframe['dimension_thing_code'] = dataframe \
                    ['dimension_thing_code'].map(int)

            if 'dimension_unity_code' in dataframe.columns:
                dataframe['dimension_unity_code'] = dataframe \
                    ['dimension_unity_code'].map(int)

            if portia_config.get('debug'):
                print('[portia-debug]: {0} rows'.format(len(dataframe.index)))

            return dataframe

        except Exception as err:
            raise Exception('couldn\'t create data frame: {0}'.format(err))

    else:
        raise Exception("couldn't retrieve data")

def convert_json(portia_config: dict, response: object) -> dict:
    """Converts a JSON text file to a data frame.
    
    Arguments:
        portia_config {dict} -- Portia's configuration arguments 
        response {object} -- HTTP response object
    
    Returns:
        dict -- converted dictionary
    
    Raises:
        Exception -- when the conversion goes wrong
    """
    if response.status_code == 200:

        json_ = json.loads(response.text)
        if portia_config['debug']:
            print('[portia-debug]: {0}'.format(json_))

        return json_

    else:
        err = json.loads(response.text)
        raise Exception(
            "couldn't retrieve data: {0}".format(err.get('message'))
        )

def convert(type_: str, portia_config: dict, response: object) -> object:
    """Converts an HTTP response.
    
    Arguments:
        type_ {str} -- type of response
        portia_config {dict} -- Portia's configuration arguments 
        response {object} -- HTTP response object
    
    Returns:
        object -- converted response
    """
    if type_ == 'text/csv':
        return convert_csv(portia_config, response)
    elif type_ == 'application/json':
        return convert_json(portia_config, response)


def plot_selection_from_dataframes(
    dataframes: list, timezone: str='Etc/GMT-3'
):
    """Uses Plotly to plot a chart from a set of dataframes.
    
    Arguments:
        dataframes {list} -- list of dataframes
    
    Keyword Arguments:
        timezone {str} -- timezone to convert the timestamp to
                          (default: {'Etc/GMT-3'})
    """
    lines = []

    for i, dataframe in enumerate(dataframes):

        dataframe['header_timestamp'] = pd.to_datetime(
            dataframe['header_timestamp'],
            unit='ms'
        ).dt.tz_localize(timezone)

        lines.append(plotlygo.Scatter(
            y=dataframe.dimension_value,
            x=dataframe.header_timestamp,
            mode='lines+markers',
            name="Sensor {0}".format(i)
        ))

    data = plotlygo.Data(lines)
    layout = plotlygo.Layout(width=1100, height=650)
    plotly.iplot(plotlygo.Figure(data=data, layout=layout))

def plot_summary_from_dataframes(
    dataframes: list, timezone: str='Etc/GMT-3'
):
    """Uses Plotly to plot a chart from a set of dataframes.
    
    Arguments:
        dataframes {list} -- list of dataframes
    
    Keyword Arguments:
        timezone {str} -- timezone to convert the timestamp to
                          (default: {'Etc/GMT-3'})
    """
    lines = []

    for dataframe in dataframes:
        dataframe['header_timestamp'] = pd.to_datetime(
            dataframe['header_timestamp'],
            unit='ms'
        ).dt.tz_localize(timezone)

        lines.append(plotlygo.Bar(
            y=dataframe['number_of_packages'],
            x=dataframe.header_timestamp,
            name="Packages",
            opacity=0.1
        ))

        if 'max' in dataframe.columns:
            lines.append(plotlygo.Scatter(
                y=dataframe['max'],
                x=dataframe.header_timestamp,
                mode='lines+markers',
                name="Max",
                yaxis='y2'
            ))

        if 'avg' in dataframe.columns:
            lines.append(plotlygo.Scatter(
                y=dataframe['avg'],
                x=dataframe.header_timestamp,
                mode='lines+markers',
                name="Avg",
                yaxis='y2'
            ))

        if 'min' in dataframe.columns:
            lines.append(plotlygo.Scatter(
                y=dataframe['min'],
                x=dataframe.header_timestamp,
                mode='lines+markers',
                name="Min",
                yaxis='y2'
            ))

        if 'median' in dataframe.columns:
            lines.append(plotlygo.Scatter(
                y=dataframe['median'],
                x=dataframe.header_timestamp,
                mode='lines+markers',
                name="Median",
                yaxis='y2'
            ))

        if 'mode' in dataframe.columns:
            lines.append(plotlygo.Scatter(
                y=dataframe['mode'],
                x=dataframe.header_timestamp,
                mode='lines+markers',
                name="Mode",
                yaxis='y2'
            ))

        if 'sum' in dataframe.columns:
            lines.append(plotlygo.Scatter(
                y=dataframe['sum'],
                x=dataframe.header_timestamp,
                mode='lines+markers',
                name="Sum",
                yaxis='y2',
                visible='legendonly'
            ))

        if 'stddev' in dataframe.columns:
            lines.append(plotlygo.Scatter(
                y=dataframe['stddev'],
                x=dataframe.header_timestamp,
                mode='lines+markers',
                name="Stddev",
                yaxis='y2',
                visible='legendonly'
            ))

        if 'spread' in dataframe.columns:
            lines.append(plotlygo.Scatter(
                y=dataframe['spread'],
                x=dataframe.header_timestamp,
                mode='lines+markers',
                name="Spread",
                yaxis='y2',
                visible='legendonly'
            ))

    data = plotlygo.Data(lines)
    layout = plotlygo.Layout(
        width=1100,
        height=650,
        yaxis=dict(title='Number of Packages', side='right'),
        yaxis2=dict(title='Dimension Value',
        overlaying='y',
        side='left'
    ))
    plotly.iplot(plotlygo.Figure(data=data, layout=layout))


def map_device_port_to_dropdown_widget(edge_id: str) -> dict:
    """Maps set of device port to a dropdown.
    
    Arguments:
        edge_id {str} -- Edge ID that identifies the device
    
    Returns:
        dict -- configuration for the dropdown
    
    Raises:
        Exception -- when something goes wrong with the request
    """
    portMapping = {}

    response = http_get_request(
        '/describe/device/{0}/ports/last?precision=ms'.format(edge_id)
    )

    if response.status_code == 200:
        d = json.loads(response.text)

        for port in d['ports']:
            label = '{0:3} | {1:25} ({2})'.format(
                port['port'],
                humanize_thing_code(port["dimension_thing_code"]),
                arrow.get(
                    port["header_timestamp"] / 1000,
                    tzinfo=tz.gettz('America/Sao_Paulo')
                ).humanize()
            )

            portMapping[label] = port['port']

        return portMapping

    else:
        raise Exception('Couldn\'t retrieve data')

def map_device_port_sensors_to_dropdown_widget(edge_id: str, port: int) -> dict:
    """Maps set of device port sensors to a dropdown.
    
    Arguments:
        edge_id {str} -- Edge ID that identifies the device
        port {int} -- port to fetch
    
    Returns:
        dict -- configuration for the dropdown
    
    Raises:
        Exception -- when something goes wrong with the request
    """
    sensorMapping = {}

    response = http_get_request(
        '/describe/device/{0}/port/{1}/sensors/last?precision=ms' \
        .format(edge_id, port)
    )

    if response.status_code == 200:
        d = json.loads(response.text)

        for sensor in d['sensors']:
            label = '{0:3} | {1:25} | {2:6}{3:15} ({4})'.format(
                sensor['sensor'],
                humanize_dimension_code(sensor["dimension_code"]),
                sensor["dimension_value"],
                humanize_unity_code(sensor["dimension_unity_code"]),
                arrow.get(
                    sensor["header_timestamp"] / 1000,
                    tzinfo=tz.gettz('America/Sao_Paulo')
                ).humanize()
            )

            sensorMapping[label] = sensor['sensor']

        return sensorMapping

    else:
        raise Exception('Couldn\'t retrieve data')

def map_device_port_sensor_dimensions_to_dropdown_widget(
    edge_id: str, port: int, sensor: int
):
    """Maps set of device port sensor dimensions to a dropdown.
    
    Arguments:
        edge_id {str} -- Edge ID that identifies the device
        port {int} -- port to fetch
        sensor {int} -- sensor to fetch
    
    Returns:
        dict -- configuration for the dropdown
    
    Raises:
        Exception -- when something goes wrong with the request
    """
    dimensionMapping = {}

    response = http_get_request(
        '/describe/device/{0}/port/{1}/sensor/{2}/dimensions/last'
        '?precision=ms'.format(edge_id, port, sensor)
    )

    if response.status_code == 200:
        d = json.loads(response.text)

        for dimension in d['dimensions']:
            label = '{0:3} | {1:25} | {2:6}{3:15} ({4})'.format(
                dimension['dimension_code'],
                humanize_dimension_code(dimension["dimension_code"]),
                dimension["dimension_value"],
                humanize_unity_code(dimension["dimension_unity_code"]),
                arrow.get(
                    dimension["header_timestamp"] / 1000,
                    tzinfo=tz.gettz('America/Sao_Paulo')
                ).humanize()
            )

            dimensionMapping[label] = dimension['dimension_code']

        return dimensionMapping

    else:
        raise Exception('Couldn\'t retrieve data')


# Maintaining compatibility with old versions
translateThingCode = humanize_thing_code
translateUnityCode = humanize_unity_code
translateDimensionCode = humanize_dimension_code



"""Unit testing of PortiaPy's utils module.
"""

import unittest

import pandas as pd

from portiapy import utils


class TestHumanization(unittest.TestCase):
	"""Set of unit tests for all functions concerning humanization of data.
	"""
	def test_humanize_thing_code(self):
		h_thing_code = utils.humanize_thing_code(1)
		self.assertEqual(h_thing_code, 'Sensor_Agriness_TU')

	def test_humanize_dimension_code(self):
		h_dimension_code = utils.humanize_dimension_code(1)
		self.assertEqual(h_dimension_code, 'Point Temperature')

		h_dimension_code = utils.humanize_dimension_code(1, 'pt-br')
		self.assertEqual(h_dimension_code, 'Temperatura Pontual')

		h_dimension_code = utils.humanize_dimension_code(1, custom={
			'en-us': { 1: 'Temperatura Pontual de Teste' }
		})
		self.assertEqual(h_dimension_code, 'Temperatura Pontual de Teste')

	def test_humanize_event_code(self):
		h_dimension_code = utils.humanize_event_code(1)
		self.assertEqual(h_dimension_code, 'Communication State')

		h_dimension_code = utils.humanize_event_code(1, 'pt-br')
		self.assertEqual(h_dimension_code, 'Estado de Comunicação')

		h_dimension_code = utils.humanize_event_code(1, custom={
			'en-us': { 1: 'Status de Comunicação' }
		})
		self.assertEqual(h_dimension_code, 'Status de Comunicação')

	def test_humanize_unity_code(self):
		h_dimension_code = utils.humanize_unity_code(1)
		self.assertEqual(h_dimension_code, '°C')

		h_dimension_code = utils.humanize_unity_code(1, 'pt-br')
		self.assertEqual(h_dimension_code, '°C')

		h_dimension_code = utils.humanize_unity_code(1, custom={
			'en-us': { 1: 'Celsius' }
		})
		self.assertEqual(h_dimension_code, 'Celsius')

	def test_humanize_dataframe(self):
		dataframe = pd.DataFrame(data=[{
			'header_timestamp': 1565634220016,
			'event_value': 'ON',
			'dimension_code': 1,
			'event_code': 1,
			'dimension_unity_code': 1,
			'dimension_thing_code': 15
		}, {
			'header_timestamp': 1565634165295,
			'event_value': 'OFF',
			'dimension_code': 3,
			'event_code': 1,
			'dimension_unity_code': 2,
			'dimension_thing_code': 16
		}])

		h_dataframe = utils.humanize_dataframe(dataframe)
		self.assertIn('header_timestamp', h_dataframe.columns)
		self.assertIn('event_value', h_dataframe.columns)
		self.assertIn('dimension_code', h_dataframe.columns)
		self.assertIn('event_code', h_dataframe.columns)
		self.assertIn('dimension_unity_code', h_dataframe.columns)
		self.assertIn('dimension_thing_code', h_dataframe.columns)
		self.assertIn('dimension_thing', h_dataframe.columns)
		self.assertIn('dimension', h_dataframe.columns)
		self.assertIn('event', h_dataframe.columns)
		self.assertIn('dimension_unity', h_dataframe.columns)

		self.assertEqual(
			h_dataframe.iloc[0].get('dimension_thing'), 'Sensor_Inobram_T'
		)
		self.assertEqual(
			h_dataframe.iloc[0].get('dimension'), 'Point Temperature'
		)
		self.assertEqual(
			h_dataframe.iloc[0].get('event'), 'Communication State'
		)
		self.assertEqual(h_dataframe.iloc[0].get('dimension_unity'), '°C')

		self.assertEqual(
			h_dataframe.iloc[1].get('dimension_thing'), 'Sensor_Inobram_TU'
		)
		self.assertEqual(
			h_dataframe.iloc[1].get('dimension'), 'Point Humidity'
		)
		self.assertEqual(
			h_dataframe.iloc[1].get('event'), 'Communication State'
		)
		self.assertEqual(h_dataframe.iloc[1].get('dimension_unity'), '%')

	def test_humanize_json(self):
		json_ = {
			'device': 'AAAABBBBCCCC',
			'channel_id': 'AAAABBBBCCCC',
			'channel_code': 14,
			'thing_code': 14,
			'ports': [{
				'port': "0",
				'thing_code': 14,
				'sensors': [{
					'sensor': '1',
					'last_package': {
						'header_timestamp': '2021-02-04T14:09:38.115Z',
						'dimension_value': 1847,
						'dimension_code': '15',
						'dimension_unity_code': 6,
						'dimension_thing_code': 14
					}
				}]
			}]
		}

		h_json = utils.humanize_json(json_)

		self.assertDictEqual(h_json, {
			'device': 'AAAABBBBCCCC',
			'channel_id': 'AAAABBBBCCCC',
			'channel_code': 14,
			'channel': 'T4_Agriness_v1',
			'thing_code': 14,
			'thing': 'T4_Agriness_v1',
			'ports': [{
				'port': "0",
				'thing_code': 14,
				'thing': 'T4_Agriness_v1',
				'sensors': [{
					'sensor': '1',
					'last_package': {
						'header_timestamp': '2021-02-04T14:09:38.115Z',
						'dimension_value': 1847,
						'dimension_code': '15',
						'dimension': 'Accumulated Time',
						'dimension_unity_code': 6,
						'dimension_unity': 's',
						'dimension_thing_code': 14,
						'dimension_thing': 'T4_Agriness_v1'
					}
				}]
			}]
		})


if __name__ == '__main__':
	unittest.main()



"""Select tools to summarize a device's data based on its ports, sensors and
dimensions.
"""

from enum import Enum

import portiapy.utils as utils


class SummaryStrategies(Enum):
    PER_MINUTE = 1
    PER_HOUR = 2
    PER_DAY = 3
    PER_MONTH = 4
    PER_YEAR = 5

    @property
    def endpoint(self):
        return self.name.lower().replace('_', '')


def query_by_dimension(
    portia_config: dict,
    dimension_code: int,
    devices: list,
    strategy: SummaryStrategies=SummaryStrategies.PER_HOUR,
    interval: int=1,
    params: dict={
        'from': None,
        'to': None,
        'lower_bound': None,
        'upper_bound': None,
        'offset': 0,
        'fill': None,
        'order': None,
        'limit': None,
        'avg': True,
        'min': True,
        'max': True,
        'sum': False,
        'median': False,
        'mode': False,
        'stddev': False,
        'spread': False,
        'last_timestamp': False,
        'precision': 'ms',
        'timezone': 'Etc/UTC'
    }) -> object:
    """Summarizes multiple devices by one dimension.
    
    Arguments:
        portia_config {dict} -- Portia's configuration arguments
        dimension_code {int} -- dimension code of the device
        devices {list} -- list of devices to summarize
    
    Keyword Arguments:
        strategy {SummaryStrategies} -- strategy to use when summarizing
                                        (default: {SummaryStrategies.PER_HOUR})
        interval {int} -- interval of time to summarize (default: {1})
        params {dict} -- params to send to the service (default: {{ 'from',
                         'to', 'lower_bound', 'upper_bound', 'offset': 0,
                         'fill', 'order', 'limit', 'min', 'max', 'sum', 'avg',
                         'median', 'mode', 'stddev', 'spread',
                         'last_timestamp', precision': 'ms',
                         'timezone': 'Etc/UTC'}})

    Returns:
        object -- object with the device's summarized dimensions
    """
    accept_header = portia_config.get('Accept')

    if accept_header is None:
        accept_header = 'text/csv'

    header = {'Accept': accept_header}

    endpoint = '/summary/dimension/{0}/{1}/{2}' \
               .format(dimension_code, strategy.endpoint, interval)

    response = utils.http_post_request(
        portia_config,
        endpoint,
        { 'devices': devices },
        params=params,
        optional_headers=header
    )

    return utils.convert(accept_header, portia_config, response)


def query_device_by_dimension(
    portia_config: dict,
    edge_id: str,
    dimension_code: int,
    series: list,
    strategy: SummaryStrategies=SummaryStrategies.PER_HOUR,
    interval: int=1,
    params: dict={
        'from': None,
        'to': None,
        'lower_bound': None,
        'upper_bound': None,
        'offset': 0,
        'fill': None,
        'order': None,
        'limit': None,
        'avg': True,
        'min': True,
        'max': True,
        'sum': False,
        'median': False,
        'mode': False,
        'stddev': False,
        'spread': False,
        'last_timestamp': False,
        'precision': 'ms',
        'timezone': 'Etc/UTC'
    }) -> object:
    """Summarizes multiple series of a device by one dimension.
    
    Arguments:
        portia_config {dict} -- Portia's configuration arguments
        edge_id {str} -- Edge ID that identifies the device
        dimension_code {int} -- dimension code of the device
        series {list} -- list of series to summarize
    
    Keyword Arguments:
        strategy {SummaryStrategies} -- strategy to use when summarizing
                                        (default: {SummaryStrategies.PER_HOUR})
        interval {int} -- interval of time to summarize (default: {1})
        params {dict} -- params to send to the service (default: {{ 'from',
                         'to', 'lower_bound', 'upper_bound', 'offset': 0,
                         'fill', 'order', 'limit', 'min', 'max', 'sum', 'avg',
                         'median', 'mode', 'stddev', 'spread',
                         'last_timestamp', precision': 'ms',
                         'timezone': 'Etc/UTC'}})

    Returns:
        object -- object with the device's summarized dimensions
    """
    accept_header = portia_config.get('Accept')

    if accept_header is None:
        accept_header = 'text/csv'

    header = {'Accept': accept_header}

    endpoint = '/summary/device/{0}/dimension/{1}/{2}/{3}' \
               .format(edge_id, dimension_code, strategy.endpoint, interval)

    response = utils.http_post_request(
        portia_config,
        endpoint,
        { 'series': series },
        params=params,
        optional_headers=header
    )

    return utils.convert(accept_header, portia_config, response)


def query_by_port_sensor(
    portia_config: dict,
    edge_id: str,
    port: int,
    sensor: int,
    strategy: SummaryStrategies=SummaryStrategies.PER_HOUR,
    interval: int=1,
    params: dict={
        'from': None,
        'to': None,
        'lower_bound': None,
        'upper_bound': None,
        'offset': 0,
        'fill': None,
        'order': None,
        'limit': None,
        'avg': True,
        'min': True,
        'max': True,
        'sum': False,
        'median': False,
        'mode': False,
        'stddev': False,
        'spread': False,
        'last_timestamp': False,
        'precision': 'ms',
        'timezone': 'Etc/UTC'
    }) -> object:
    """Summarizes a device by port and sensor.
    
    Arguments:
        portia_config {dict} -- Portia's configuration arguments
        edge_id {str} -- Edge ID that identifies the device
        port {int} -- port of the device
        sensor {int} -- sensor of the device
    
    Keyword Arguments:
        strategy {SummaryStrategies} -- strategy to use when summarizing
                                        (default: {SummaryStrategies.PER_HOUR})
        interval {int} -- interval of time to summarize (default: {1})
        params {dict} -- params to send to the service (default: {{ 'from',
                         'to', 'lower_bound', 'upper_bound', 'offset': 0,
                         'fill', 'order', 'limit', 'min', 'max', 'sum', 'avg',
                         'median', 'mode', 'stddev', 'spread',
                         'last_timestamp', precision': 'ms',
                         'timezone': 'Etc/UTC'}})

    Returns:
        object -- object with the device's summarized dimensions
    """
    accept_header = portia_config.get('Accept')

    if accept_header is None:
        accept_header = 'text/csv'

    header = {'Accept': accept_header}

    endpoint = '/summary/device/{0}/port/{1}/sensor/{2}/{3}/{4}' \
               .format(edge_id, port, sensor, strategy.endpoint, interval)

    response = utils.http_get_request(
        portia_config, endpoint, params=params, optional_headers=header
    )

    return utils.convert(accept_header, portia_config, response)


def query_by_port_sensor_dimension(
    portia_config: dict,
    edge_id: str,
    port: int,
    sensor: int,
    dimension_code: int,
    strategy: SummaryStrategies=SummaryStrategies.PER_HOUR,
    interval: int=1,
    params: dict={
        'from': None,
        'to': None,
        'lower_bound': None,
        'upper_bound': None,
        'offset': 0,
        'fill': None,
        'order': None,
        'limit': None,
        'avg': True,
        'min': True,
        'max': True,
        'sum': False,
        'median': False,
        'mode': False,
        'stddev': False,
        'spread': False,
        'last_timestamp': False,
        'precision': 'ms',
        'timezone': 'Etc/UTC'
    }) -> object:
    """Summarizes a device by port, sensor and dimension code.
    
    Arguments:
        portia_config {dict} -- Portia's configuration arguments
        edge_id {str} -- Edge ID that identifies the device
        port {int} -- port of the device
        sensor {int} -- sensor of the device
        dimension_code {int} -- dimension code of the device
    
    Keyword Arguments:
        strategy {SummaryStrategies} -- strategy to use when summarizing
                                        (default: {SummaryStrategies.PER_HOUR})
        interval {int} -- interval of time to summarize (default: {1})
        params {dict} -- params to send to the service (default: {{ 'from',
                         'to', 'lower_bound', 'upper_bound', 'offset': 0,
                         'fill', 'order', 'limit', 'min', 'max', 'sum', 'avg',
                         'median', 'mode', 'stddev', 'spread',
                         'last_timestamp', precision': 'ms',
                         'timezone': 'Etc/UTC'}})

    Returns:
        object -- object with the device's summarized dimensions
    """
    accept_header = portia_config.get('Accept')

    if accept_header is None:
        accept_header = 'text/csv'

    header = {'Accept': accept_header}

    endpoint = ('/summary/device/{0}/port/{1}/sensor/{2}/dimension/{3}/{4}'
                '/{5}'.format(
                    edge_id,
                    port,
                    sensor,
                    dimension_code,
                    strategy.endpoint,
                    interval
                ))

    response = utils.http_get_request(
        portia_config, endpoint, params=params, optional_headers=header
    )

    return utils.convert(accept_header, portia_config, response)



"""Integration testing for the application's summary module.
"""

import os
import unittest

from portiapy import summary


from dotenv import load_dotenv, find_dotenv
load_dotenv(find_dotenv())


class TestSummary(unittest.TestCase):
	"""Set of integration tests for all functions concerning the summary
	module.
	"""
	@classmethod
	def setUpClass(cls):
		"""Creates a Portia configuration to test the adapter.
		"""

		# Settings for test
		base_url = os.getenv('PORTIA_URL', 'https://api-portia.agriness.io/v3')
		token = os.getenv('PORTIA_TOKEN')

		# Creating portia configuration
		cls.portia_config = {
			'baseurl': base_url,
			'authorization': token,
			'debug': False,
			'Accept': 'text/csv'
		}

	def test_summary_strategies(self):
		per_minute = summary.SummaryStrategies.PER_MINUTE
		per_hour = summary.SummaryStrategies.PER_HOUR
		per_day = summary.SummaryStrategies.PER_DAY
		per_month = summary.SummaryStrategies.PER_MONTH
		per_year = summary.SummaryStrategies.PER_YEAR
		
		self.assertEqual(per_minute.endpoint, 'perminute')
		self.assertEqual(per_hour.endpoint, 'perhour')
		self.assertEqual(per_day.endpoint, 'perday')
		self.assertEqual(per_month.endpoint, 'permonth')
		self.assertEqual(per_year.endpoint, 'peryear')

	def test_query_by_dimension(self):
		summarized_dimensions = summary.query_by_dimension(
			self.portia_config,
			1,
			[{
				'device': '2DPEQ572HEXP'
			}, {
				'device': '3WPH414SRSTZ'
			}],
			strategy=summary.SummaryStrategies.PER_DAY,
			params={
		        'from': 1609470000000,
		        'to': 1609729199000,
		        'lower_bound': None,
		        'upper_bound': None,
		        'offset': 0,
		        'fill': None,
		        'order': None,
		        'limit': None,
		        'avg': True,
		        'min': True,
		        'max': True,
		        'sum': False,
		        'median': False,
		        'mode': False,
		        'stddev': False,
		        'spread': False,
		        'last_timestamp': False,
		        'precision': 'ms',
		        'timezone': 'Etc/UTC'
		    }
		)

		for i, row in summarized_dimensions.iterrows():
			self.assertEqual(row.dimension_unity_code, 1)

	def test_query_device_by_dimension(self):
		summarized_dimensions = summary.query_device_by_dimension(
			self.portia_config,
			'2DPEQ572HEXP',
			1,
			None,
			strategy=summary.SummaryStrategies.PER_DAY,
			params={
		        'from': 1609470000000,
		        'to': 1609729199000,
		        'lower_bound': None,
		        'upper_bound': None,
		        'offset': 0,
		        'fill': None,
		        'order': None,
		        'limit': None,
		        'avg': True,
		        'min': True,
		        'max': True,
		        'sum': False,
		        'median': False,
		        'mode': False,
		        'stddev': False,
		        'spread': False,
		        'last_timestamp': False,
		        'precision': 'ms',
		        'timezone': 'Etc/UTC'
		    }
		)

		for i, row in summarized_dimensions.iterrows():
			self.assertEqual(row.dimension_unity_code, 1)

	def test_query_by_port_sensor(self):
		summarized_dimensions = summary.query_by_port_sensor(
			self.portia_config,
			'2DPEQ572HEXP',
			4, 
			1,
			strategy=summary.SummaryStrategies.PER_DAY,
			params={
		        'from': 1609470000000,
		        'to': 1609729199000,
		        'lower_bound': None,
		        'upper_bound': None,
		        'offset': 0,
		        'fill': None,
		        'order': None,
		        'limit': None,
		        'avg': True,
		        'min': True,
		        'max': True,
		        'sum': False,
		        'median': False,
		        'mode': False,
		        'stddev': False,
		        'spread': False,
		        'last_timestamp': False,
		        'precision': 'ms',
		        'timezone': 'Etc/UTC'
		    }
		)

		for i, row in summarized_dimensions.iterrows():
			self.assertEqual(row.dimension_unity_code, 1)

	def test_query_by_port_sensor_dimension(self):
		summarized_dimensions = summary.query_by_port_sensor_dimension(
			self.portia_config,
			'2DPEQ572HEXP',
			4,
			1,
			1,
			strategy=summary.SummaryStrategies.PER_DAY,
			params={
		        'from': 1609470000000,
		        'to': 1609729199000,
		        'lower_bound': None,
		        'upper_bound': None,
		        'offset': 0,
		        'fill': None,
		        'order': None,
		        'limit': None,
		        'avg': True,
		        'min': True,
		        'max': True,
		        'sum': False,
		        'median': False,
		        'mode': False,
		        'stddev': False,
		        'spread': False,
		        'last_timestamp': False,
		        'precision': 'ms',
		        'timezone': 'Etc/UTC'
		    }
		)

		for i, row in summarized_dimensions.iterrows():
			self.assertEqual(row.dimension_unity_code, 1)


if __name__ == '__main__':
	unittest.main()



"""Integration testing for the application's select module.
"""

import os
import unittest

from portiapy import events


from dotenv import load_dotenv, find_dotenv
load_dotenv(find_dotenv())


class TestEvents(unittest.TestCase):
	"""Set of integration tests for all functions concerning the events
	module.
	"""
	@classmethod
	def setUpClass(cls):
		"""Creates a Portia configuration to test the adapter.
		"""

		# Settings for test
		base_url = os.getenv('PORTIA_URL', 'https://api-portia.agriness.io/v3')
		token = os.getenv('PORTIA_TOKEN')

		# Creating portia configuration
		cls.portia_config = {
			'baseurl': base_url,
			'authorization': token,
			'debug': False,
			'Accept': 'text/csv'
		}

	def test_query_by_port_sensor(self):
		device_events = events.query_by_port_sensor(
			self.portia_config, '2DPEQ572HEXP', 4, 1
		)

		for i, row in device_events.iterrows():
			self.assertEqual(row.dimension_thing_code, 16)
			self.assertEqual(row.dimension_unity_code, 1)
			self.assertEqual(row.dimension_code, 1)
			self.assertEqual(row.event_code, 1)

	def test_query_by_port_sensor_last(self):
		device_events = events.query_by_port_sensor(
			self.portia_config, '2DPEQ572HEXP', 4, 1, last=True
		)

		self.assertEqual(device_events.iloc[0].dimension_thing_code, 16)
		self.assertEqual(device_events.iloc[0].dimension_unity_code, 1)
		self.assertEqual(device_events.iloc[0].dimension_code, 1)
		self.assertEqual(device_events.iloc[0].event_code, 1)

	def test_query_by_port_dimension(self):
		device_events = events.query_by_port_dimension(
			self.portia_config, '2DPEQ572HEXP', 4, 1
		)

		for i, row in device_events.iterrows():
			self.assertEqual(row.dimension_thing_code, 16)
			self.assertEqual(row.dimension_unity_code, 1)
			self.assertEqual(row.dimension_code, 1)
			self.assertEqual(row.event_code, 1)

	def test_query_by_port_dimension_last(self):
		device_events = events.query_by_port_dimension(
			self.portia_config, '2DPEQ572HEXP', 4, 1, last=True
		)

		self.assertEqual(device_events.iloc[0].dimension_thing_code, 16)
		self.assertEqual(device_events.iloc[0].dimension_unity_code, 1)
		self.assertEqual(device_events.iloc[0].dimension_code, 1)
		self.assertEqual(device_events.iloc[0].event_code, 1)

	def test_query_by_port_sensor_dimension(self):
		device_events = events.query_by_port_sensor_dimension(
			self.portia_config, '2DPEQ572HEXP', 4, 1, 1
		)

		for i, row in device_events.iterrows():
			self.assertEqual(row.dimension_thing_code, 16)
			self.assertEqual(row.dimension_unity_code, 1)
			self.assertEqual(row.dimension_code, 1)
			self.assertEqual(row.event_code, 1)

	def test_query_by_port_sensor_dimension_last(self):
		device_events = events.query_by_port_sensor_dimension(
			self.portia_config, '2DPEQ572HEXP', 4, 1, 1, last=True
		)

		self.assertEqual(device_events.iloc[0].dimension_thing_code, 16)
		self.assertEqual(device_events.iloc[0].dimension_unity_code, 1)
		self.assertEqual(device_events.iloc[0].dimension_code, 1)
		self.assertEqual(device_events.iloc[0].event_code, 1)

	def test_query_by_port_sensor_event(self):
		device_events = events.query_by_port_sensor_event(
			self.portia_config, '2DPEQ572HEXP', 4, 1, 1
		)

		for i, row in device_events.iterrows():
			self.assertEqual(row.dimension_thing_code, 16)
			self.assertEqual(row.dimension_unity_code, 1)
			self.assertEqual(row.dimension_code, 1)
			self.assertEqual(row.event_code, 1)

	def test_query_by_port_sensor_event_last(self):
		device_events = events.query_by_port_sensor_event(
			self.portia_config, '2DPEQ572HEXP', 4, 1, 1, last=True
		)

		self.assertEqual(device_events.iloc[0].dimension_thing_code, 16)
		self.assertEqual(device_events.iloc[0].dimension_unity_code, 1)
		self.assertEqual(device_events.iloc[0].dimension_code, 1)
		self.assertEqual(device_events.iloc[0].event_code, 1)

	def test_query_by_port_dimension_event(self):
		device_events = events.query_by_port_dimension_event(
			self.portia_config, '2DPEQ572HEXP', 4, 1, 1
		)

		for i, row in device_events.iterrows():
			self.assertEqual(row.dimension_thing_code, 16)
			self.assertEqual(row.dimension_unity_code, 1)
			self.assertEqual(row.dimension_code, 1)
			self.assertEqual(row.event_code, 1)

	def test_query_by_port_dimension_event_last(self):
		device_events = events.query_by_port_dimension_event(
			self.portia_config, '2DPEQ572HEXP', 4, 1, 1, last=True
		)

		self.assertEqual(device_events.iloc[0].dimension_thing_code, 16)
		self.assertEqual(device_events.iloc[0].dimension_unity_code, 1)
		self.assertEqual(device_events.iloc[0].dimension_code, 1)
		self.assertEqual(device_events.iloc[0].event_code, 1)

	def test_query_by_port_sensor_dimension_event(self):
		device_events = events.query_by_port_sensor_dimension_event(
			self.portia_config, '2DPEQ572HEXP', 4, 1, 1, 1
		)

		for i, row in device_events.iterrows():
			self.assertEqual(row.dimension_thing_code, 16)
			self.assertEqual(row.dimension_unity_code, 1)
			self.assertEqual(row.dimension_code, 1)
			self.assertEqual(row.event_code, 1)

	def test_query_by_port_sensor_dimension_event_last(self):
		device_events = events.query_by_port_sensor_dimension_event(
			self.portia_config, '2DPEQ572HEXP', 4, 1, 1, 1, last=True
		)

		self.assertEqual(device_events.iloc[0].dimension_thing_code, 16)
		self.assertEqual(device_events.iloc[0].dimension_unity_code, 1)
		self.assertEqual(device_events.iloc[0].dimension_code, 1)
		self.assertEqual(device_events.iloc[0].event_code, 1)


if __name__ == '__main__':
	unittest.main()



#####################################
#               Phases              #
#####################################

# Libraries
import json                       # JSON encoder and decoder
import portiapy.utils as utils

# Functions
def about():
    print("portiapy.describe - an Agriness Edge project")

########################################
# index
########################################
# /pipeline/phases
########################################
def index(portiaConfig):

	endpoint = '/pipeline/phases'
	response = utils.httpGetRequest(portiaConfig, endpoint)

	if response.status_code == 200:

		d = json.loads(response.text)
		if portiaConfig['debug']:
			print( '[portia-debug]: {0}'.format(d) )

		return d

	else:

		d = json.loads(response.text)
		raise Exception( 'couldn\'t retrieve data: {0}'.format(d['message']) )

########################################
# store
########################################
# /pipeline/phases
########################################
def store(portiaConfig, payload):

	endpoint = '/pipeline/phases'
	response = utils.httpPostRequest(portiaConfig, endpoint, payload)

	if response.status_code == 200:

		d = json.loads(response.text)
		if portiaConfig['debug']:
			print( '[portia-debug]: {0}'.format(d) )

		return d

	else:

		d = json.loads(response.text)
		raise Exception( 'couldn\'t retrieve data: {0}'.format(d['message']) )

########################################
# show
########################################
# /pipeline/phases/:phase
########################################
def show(portiaConfig, phaseName):

	endpoint = '/pipeline/phases/{0}'.format(phaseName)
	response = utils.httpGetRequest(portiaConfig, endpoint)

	if response.status_code == 200:

		d = json.loads(response.text)
		if portiaConfig['debug']:
			print( '[portia-debug]: {0}'.format(d) )

		return d

	else:

		d = json.loads(response.text)
		raise Exception( 'couldn\'t retrieve data: {0}'.format(d['message']) )

########################################
# update
########################################
# /pipeline/phases/:phase
########################################
def update(portiaConfig, phaseName, payload):

	endpoint = '/pipeline/phases/{0}'.format(phaseName)
	response = utils.httpPutRequest(portiaConfig, endpoint, payload)

	if response.status_code == 200:

		d = json.loads(response.text)
		if portiaConfig['debug']:
			print( '[portia-debug]: {0}'.format(d) )

		return d

	else:

		d = json.loads(response.text)
		raise Exception( 'couldn\'t retrieve data: {0}'.format(d['message']) )



"""Integration testing for the application's describe module.
"""

import os
import unittest

import pandas as pd

from portiapy import describe


from dotenv import load_dotenv, find_dotenv
load_dotenv(find_dotenv())


class TestDescribe(unittest.TestCase):
	"""Set of integration tests for all functions concerning the describe
	module.
	"""
	@classmethod
	def setUpClass(cls):
		"""Creates a Portia configuration to test the adapter.
		"""

		# Settings for test
		base_url = os.getenv('PORTIA_URL', 'https://api-portia.agriness.io/v3')
		token = os.getenv('PORTIA_TOKEN')

		# Creating portia configuration
		cls.portia_config = {
			'baseurl': base_url,
			'authorization': token,
			'debug': False,
			'Accept': 'text/csv'
		}

	def test_device_ports(self):
		ports = describe.device_ports(self.portia_config, '2DPEQ572HEXP')
		self.assertListEqual(ports, [0, 1, 2, 4, 5, 8, 9, 10, 11, 12, 13])

	def test_device_ports_last(self):
		ports = describe.device_ports(
			self.portia_config, '2DPEQ572HEXP', last=True
		)

		self.assertListEqual(
			ports['port'].tolist(), [0, 1, 2, 4, 5, 8, 9, 10, 11, 12, 13]
		)

		self.assertListEqual(
			ports['dimension_thing_code'].tolist(),
			[69, 15, 15, 16, 16, 22, 22, 22, 23, 68, 68]
		)

	def test_device_port_sensors(self):
		sensors = describe.device_port_sensors(
			self.portia_config, '2DPEQ572HEXP', 1
		)

		self.assertListEqual(sensors, [1])

	def test_device_port_sensors_last(self):
		sensors = describe.device_port_sensors(
			self.portia_config, '2DPEQ572HEXP', 1, last=True
		)

		self.assertEqual(sensors.iloc[0]['sensor'], 1)
		self.assertEqual(sensors.iloc[0]['dimension_code'], 1)
		self.assertEqual(sensors.iloc[0]['dimension_unity_code'], 1)
		self.assertEqual(sensors.iloc[0]['dimension_thing_code'], 15)

	def test_device_port_dimensions(self):
		dimensions = describe.device_port_dimensions(
			self.portia_config, '2DPEQ572HEXP', 1
		)

		self.assertListEqual(dimensions, [1, 9])

	def test_device_port_dimensions_last(self):
		dimensions = describe.device_port_dimensions(
			self.portia_config, '2DPEQ572HEXP', 1, last=True
		)

		self.assertEqual(dimensions.iloc[0]['sensor'], 1)
		self.assertEqual(dimensions.iloc[0]['dimension_code'], 1)
		self.assertEqual(dimensions.iloc[0]['dimension_thing_code'], 15)

		self.assertEqual(dimensions.iloc[1]['sensor'], 1)
		self.assertEqual(dimensions.iloc[1]['dimension_code'], 9)
		self.assertEqual(dimensions.iloc[1]['dimension_thing_code'], 15)

	def test_device_port_sensor_dimensions(self):
		dimensions = describe.device_port_sensor_dimensions(
			self.portia_config, '2DPEQ572HEXP', 1, 1
		)

		self.assertListEqual(dimensions, [1, 9])

	def test_device_port_sensor_dimensions_last(self):
		dimensions = describe.device_port_sensor_dimensions(
			self.portia_config, '2DPEQ572HEXP', 1, 1, last=True
		)

		self.assertEqual(dimensions.iloc[0]['dimension_code'], 1)
		self.assertEqual(dimensions.iloc[0]['dimension_unity_code'], 1)
		self.assertEqual(dimensions.iloc[0]['dimension_thing_code'], 15)

		self.assertEqual(dimensions.iloc[1]['dimension_code'], 9)
		self.assertEqual(dimensions.iloc[1]['dimension_unity_code'], 1)
		self.assertEqual(dimensions.iloc[1]['dimension_thing_code'], 15)


if __name__ == '__main__':
	unittest.main()



"""Description tools to discover a device's list of ports, sensors and
dimensions.
"""

import json

import pandas as pd

import portiapy.utils as utils


def device_ports(
    portia_config: dict,
    edge_id: str,
    last: bool=False,
    params: dict={
        'from': None,
        'to': None,
        'sort': True,
        'precision': 'ms',
        'timezone': 'Etc/UTC'
}) -> object:
    """Lists a device's ports.
    
    Arguments:
        portia_config {dict} -- Portia's configuration arguments 
        edge_id {str} -- Edge ID that identifies the device
    
    Keyword Arguments:
        last {bool} -- if the last package of each port should be returned or
                       not (default: {False})
        params {dict} -- params to send to the service (default:
                         {{ 'from', 'to', 'sort', 'precision': 'ms',
                         'timezone': 'Etc/UTC' }})
    
    Returns:
        object -- object with the list of ports
    
    Raises:
        Exception -- when the request goes wrong
    """
    if last == False:
        endpoint = '/describe/device/{0}/ports'.format(edge_id)
    else:
        endpoint = '/describe/device/{0}/ports/last'.format(edge_id)

    response = utils.http_get_request(portia_config, endpoint, params)

    if response.status_code == 200:

        d = json.loads(response.text).get('ports')

        if portia_config.get('debug'):
            print('[portia-debug]: {0}'.format(d))

        if last == True:
            d = pd.DataFrame(
                d,
                columns=['header_timestamp', 'port', 'dimension_thing_code']
            )

            d['port'] = d['port'].map(int)
            d['dimension_thing_code'] = d['dimension_thing_code'].map(int)
        else:
            d = list(map(int, d))

        return d

    else:
        raise Exception("couldn't retrieve data")

def device_port_sensors(
    portia_config: dict,
    edge_id: str,
    port: int,
    last: bool=False,
    params: dict={
        'from': None,
        'to': None,
        'sort': True,
        'precision': 'ms',
        'timezone': 'Etc/UTC'
}) -> object:
    """Lists a port's sensors.
    
    Arguments:
        portia_config {dict} -- Portia's configuration arguments
        edge_id {str} -- Edge ID that identifies the device
        port {int} -- port of the device
    
    Keyword Arguments:
        last {bool} -- if the last package of each port should be returned or
                       not (default: {False})
        params {dict} -- params to send to the service (default:
                         {{ 'from', 'to', 'sort', 'precision': 'ms',
                         'timezone': 'Etc/UTC' }})
    
    Returns:
        object -- object with the list of sensors
    
    Raises:
        Exception -- when the request goes wrong
    """
    if last == False:
        endpoint = '/describe/device/{0}/port/{1}/sensors' \
                   .format(edge_id, port)
    else:
        endpoint = '/describe/device/{0}/port/{1}/sensors/last' \
                   .format(edge_id, port)

    response = utils.http_get_request(portia_config, endpoint, params)

    if response.status_code == 200:

        d = json.loads(response.text).get('sensors')
        if portia_config.get('debug'):
            print('[portia-debug]: {0}'.format(d))

        if last == True:
            d = pd.DataFrame(
                d,
                columns=[
                    'header_timestamp',
                    'sensor',
                    'dimension_value',
                    'dimension_code',
                    'dimension_unity_code',
                    'dimension_thing_code'
                ]
            )

            d['sensor'] = d['sensor'].map(int)
            d['dimension_code'] = d['dimension_code'].map(int)
            d['dimension_unity_code'] = d['dimension_unity_code'].map(int)
            d['dimension_thing_code'] = d['dimension_thing_code'].map(int)
        else:
            d = list(map(int, d))

        return d

    else:
        raise Exception("couldn't retrieve data")

def device_port_dimensions(
    portia_config: dict,
    edge_id: str,
    port: int,
    last: bool=False,
    params: dict={
        'from': None,
        'to': None,
        'sort': True,
        'precision': 'ms',
        'timezone': 'Etc/UTC'
}) -> object:
    """Lists a port's dimensions.
    
    Arguments:
        portia_config {dict} -- Portia's configuration arguments
        edge_id {str} -- Edge ID that identifies the device
        port {int} -- port of the device
    
    Keyword Arguments:
        last {bool} -- if the last package of each port should be returned or
                       not (default: {False})
        params {dict} -- params to send to the service (default:
                         {{ 'from', 'to', 'sort', 'precision': 'ms',
                         'timezone': 'Etc/UTC' }})
    
    Returns:
        object -- object with the list of dimensions
    
    Raises:
        Exception -- when the request goes wrong
    """
    if last == False:
        endpoint = '/describe/device/{0}/port/{1}/dimensions' \
                   .format(edge_id, port)
    else:
        endpoint = '/describe/device/{0}/port/{1}/dimensions/last' \
                   .format(edge_id, port)

    response = utils.http_get_request(portia_config, endpoint, params)

    if response.status_code == 200:

        d = json.loads(response.text).get('dimensions')
        if portia_config.get('debug'):
            print('[portia-debug]: {0}'.format(d))

        if last == True:
            d = pd.DataFrame(
                d,
                columns=[
                    'header_timestamp',
                    'dimension_code',
                    'sensor',
                    'dimension_thing_code'
                ]
            )

            d['sensor'] = d['sensor'].map(int)
            d['dimension_code'] = d['dimension_code'].map(int)
            d['dimension_thing_code'] = d['dimension_thing_code'].map(int)
        else:
            d = list(map(int, d))

        return d

    else:
        raise Exception("couldn't retrieve data")

def device_port_sensor_dimensions(
    portia_config: dict,
    edge_id: str,
    port: int,
    sensor: int,
    last: bool=False,
    params: dict={
        'from': None,
        'to': None,
        'sort': True,
        'precision': 'ms',
        'timezone': 'Etc/UTC'
}) -> object:
    """Lists a sensor's dimensions.
    
    Arguments:
        portia_config {dict} -- Portia's configuration arguments
        edge_id {str} -- Edge ID that identifies the device
        port {int} -- port of the device
        sensor {int} -- sensor of the device
    
    Keyword Arguments:
        last {bool} -- if the last package of each port should be returned or
                       not (default: {False})
        params {dict} -- params to send to the service (default:
                         {{ 'from', 'to', 'sort', 'precision': 'ms',
                         'timezone': 'Etc/UTC' }})
    
    Returns:
        object -- object with the list of dimensions
    
    Raises:
        Exception -- when the request goes wrong
    """
    if last == False:
        endpoint = '/describe/device/{0}/port/{1}/sensor/{2}/dimensions' \
                   .format(edge_id, port, sensor)
    else:
        endpoint = '/describe/device/{0}/port/{1}/sensor/{2}/dimensions/last' \
                   .format(edge_id, port, sensor)

    response = utils.http_get_request(portia_config, endpoint, params)

    if response.status_code == 200:

        d = json.loads(response.text).get('dimensions')
        if portia_config.get('debug'):
            print('[portia-debug]: {0}'.format(d))

        if last == True:
            d = pd.DataFrame(
                d,
                columns=[
                    'header_timestamp',
                    'dimension_value',
                    'dimension_code',
                    'dimension_unity_code',
                    'dimension_thing_code'
                ]
            )

            d['dimension_code'] = d['dimension_code'].map(int)
            d['dimension_unity_code'] = d['dimension_unity_code'].map(int)
            d['dimension_thing_code'] = d['dimension_thing_code'].map(int)
        else:
            d = list(map(int, d))

        return d

    else:
        raise Exception("couldn't retrieve data")



"""Integration testing for the application's portia module.
"""

import os
import unittest

from portiapy import portia


from dotenv import load_dotenv, find_dotenv
load_dotenv(find_dotenv())


class TestEdgeDevice(unittest.TestCase):
	"""Set of integration tests for all functions concerning the EdgeDevice
	instance.
	"""
	@classmethod
	def setUpClass(cls):
		"""Creates a Portia configuration to test the instance.
		"""

		# Settings for test
		base_url = os.getenv('PORTIA_URL', 'https://api-portia.agriness.io/v3')
		token = os.getenv('PORTIA_TOKEN')

		# Creating Portia API instance
		cls.portia_api = portia.PortiaApi({
			'baseurl': base_url,
			'authorization': token,
			'debug': False,
			'Accept': 'text/csv'
		})

	def test_ports(self):
		ports = self.portia_api.device('2DPEQ572HEXP').ports(params={
	        'from': None,
	        'to': None,
	        'sort': True,
	        'precision': 'ms',
	        'timezone': 'Etc/UTC'
		})
		self.assertListEqual(ports, [0, 1, 2, 4, 5, 8, 9, 10, 11, 12, 13])

	def test_ports_last(self):
		ports = self.portia_api.device('2DPEQ572HEXP').ports(
			last=True,
			params={
		        'from': None,
		        'to': None,
		        'sort': True,
		        'precision': 'ms',
		        'timezone': 'Etc/UTC'
			}
		)

		self.assertListEqual(
			ports['port'].tolist(), [0, 1, 2, 4, 5, 8, 9, 10, 11, 12, 13]
		)

		self.assertListEqual(
			ports['dimension_thing_code'].tolist(),
			[69, 15, 15, 16, 16, 22, 22, 22, 23, 68, 68]
		)

	def test_profile(self):
		device = self.portia_api.device('2DPEQ572HEXP').profile(params={
			'sort': True,
			'precision': 'ms',
			'timezone': 'Etc/UTC'
		})

		self.assertEqual(device.get('device'), '2DPEQ572HEXP')
		self.assertEqual(device.get('channel_id'), '1BTU7542G5EY')
		self.assertEqual(device.get('channel_code'), 14)
		self.assertEqual(device.get('thing_code'), 69)
		self.assertEqual(len(device.get('ports')), 9)


class TestEdgeDevicePort(unittest.TestCase):
	"""Set of integration tests for all functions concerning the EdgeDevicePort
	instance.
	"""
	@classmethod
	def setUpClass(cls):
		"""Creates a Portia configuration to test the instance.
		"""

		# Settings for test
		base_url = os.getenv('PORTIA_URL', 'https://api-portia.agriness.io/v3')
		token = os.getenv('PORTIA_TOKEN')

		# Creating Portia API instance
		cls.portia_api = portia.PortiaApi({
			'baseurl': base_url,
			'authorization': token,
			'debug': False,
			'Accept': 'text/csv'
		})

	def test_sensors(self):
		sensors = self.portia_api.device('2DPEQ572HEXP').port(1).sensors(
			params={
		        'from': None,
		        'to': None,
		        'sort': True,
		        'precision': 'ms',
		        'timezone': 'Etc/UTC'
			}
		)
		self.assertListEqual(sensors, [1])

	def test_sensors_last(self):
		sensors = self.portia_api.device('2DPEQ572HEXP').port(1).sensors(
			last=True,
			params={
		        'from': None,
		        'to': None,
		        'sort': True,
		        'precision': 'ms',
		        'timezone': 'Etc/UTC'
			}
		)

		self.assertEqual(sensors.iloc[0]['sensor'], 1)
		self.assertEqual(sensors.iloc[0]['dimension_code'], 1)
		self.assertEqual(sensors.iloc[0]['dimension_unity_code'], 1)
		self.assertEqual(sensors.iloc[0]['dimension_thing_code'], 15)

		humanized_sensors = sensors.humanize()
		self.assertEqual(humanized_sensors.iloc[0]['sensor'], 1)
		self.assertEqual(
			humanized_sensors.iloc[0]['dimension_code'], 1
		)
		self.assertEqual(
			humanized_sensors.iloc[0]['dimension'], 'Point Temperature'
		)
		self.assertEqual(
			humanized_sensors.iloc[0]['dimension_unity_code'], 1
		)
		self.assertEqual(
			humanized_sensors.iloc[0]['dimension_unity'], '°C'
		)
		self.assertEqual(
			humanized_sensors.iloc[0]['dimension_thing_code'], 15
		)
		self.assertEqual(
			humanized_sensors.iloc[0]['dimension_thing'], 'Sensor_Inobram_T'
		)

	def test_dimensions(self):
		dimensions = self.portia_api.device('2DPEQ572HEXP').port(1).dimensions(
			params={
		        'from': None,
		        'to': None,
		        'sort': True,
		        'precision': 'ms',
		        'timezone': 'Etc/UTC'
			}
		)
		self.assertListEqual(dimensions, [1, 9])

	def test_dimensions_last(self):
		dimensions = self.portia_api.device('2DPEQ572HEXP').port(1).dimensions(
			last=True,
			params={
		        'from': None,
		        'to': None,
		        'sort': True,
		        'precision': 'ms',
		        'timezone': 'Etc/UTC'
			}
		)

		self.assertEqual(dimensions.iloc[0]['sensor'], 1)
		self.assertEqual(dimensions.iloc[0]['dimension_code'], 1)
		self.assertEqual(dimensions.iloc[0]['dimension_thing_code'], 15)

		self.assertEqual(dimensions.iloc[1]['sensor'], 1)
		self.assertEqual(dimensions.iloc[1]['dimension_code'], 9)
		self.assertEqual(dimensions.iloc[1]['dimension_thing_code'], 15)

		humanized_dimensions = dimensions.humanize()
		self.assertEqual(humanized_dimensions.iloc[0]['sensor'], 1)
		self.assertEqual(
			humanized_dimensions.iloc[0]['dimension_code'], 1
		)
		self.assertEqual(
			humanized_dimensions.iloc[0]['dimension'], 'Point Temperature'
		)
		self.assertEqual(
			humanized_dimensions.iloc[0]['dimension_thing_code'], 15
		)
		self.assertEqual(
			humanized_dimensions.iloc[0]['dimension_thing'], 'Sensor_Inobram_T'
		)

		self.assertEqual(humanized_dimensions.iloc[1]['sensor'], 1)
		self.assertEqual(
			humanized_dimensions.iloc[1]['dimension_code'], 9
		)
		self.assertEqual(
			humanized_dimensions.iloc[1]['dimension'],
			'Point Water Temperature'
		)
		self.assertEqual(
			humanized_dimensions.iloc[1]['dimension_thing_code'], 15
		)
		self.assertEqual(
			humanized_dimensions.iloc[1]['dimension_thing'], 'Sensor_Inobram_T'
		)

	def test_profile(self):
		port = self.portia_api.device('2DPEQ572HEXP').port(4).profile(params={
			'sort': True,
			'precision': 'ms',
			'timezone': 'Etc/UTC'
		})

		self.assertEqual(port.get('device'), '2DPEQ572HEXP')
		self.assertEqual(port.get('channel_id'), '1BTU7542G5EY')
		self.assertEqual(port.get('channel_code'), 14)
		self.assertEqual(port.get('thing_code'), 69)
		self.assertEqual(len(port.get('ports')), 1)
		self.assertEqual(len(port.get('ports')[0].get('sensors')), 2)


class TestEdgeDeviceSensor(unittest.TestCase):
	"""Set of integration tests for all functions concerning the
	EdgeDeviceSensor instance.
	"""
	@classmethod
	def setUpClass(cls):
		"""Creates a Portia configuration to test the instance.
		"""

		# Settings for test
		base_url = os.getenv('PORTIA_URL', 'https://api-portia.agriness.io/v3')
		token = os.getenv('PORTIA_TOKEN')

		# Creating Portia API instance
		cls.portia_api = portia.PortiaApi({
			'baseurl': base_url,
			'authorization': token,
			'debug': False,
			'Accept': 'text/csv'
		})

	def test_dimensions(self):
		dimensions = self.portia_api.device('2DPEQ572HEXP').port(1).sensor(1) \
			.dimensions(
				params={
			        'from': None,
			        'to': None,
			        'sort': True,
			        'precision': 'ms',
			        'timezone': 'Etc/UTC'
				}
			)
		self.assertListEqual(dimensions, [1, 9])

	def test_dimensions_last(self):
		dimensions = self.portia_api.device('2DPEQ572HEXP').port(1).sensor(1) \
			.dimensions(
				last=True,
				params={
			        'from': None,
			        'to': None,
			        'sort': True,
			        'precision': 'ms',
			        'timezone': 'Etc/UTC'
				}
			)

		self.assertEqual(dimensions.iloc[0]['dimension_code'], 1)
		self.assertEqual(dimensions.iloc[0]['dimension_unity_code'], 1)
		self.assertEqual(dimensions.iloc[0]['dimension_thing_code'], 15)

		self.assertEqual(dimensions.iloc[1]['dimension_code'], 9)
		self.assertEqual(dimensions.iloc[1]['dimension_unity_code'], 1)
		self.assertEqual(dimensions.iloc[1]['dimension_thing_code'], 15)

		humanized_dimensions = dimensions.humanize()
		self.assertEqual(
			humanized_dimensions.iloc[0]['dimension_code'], 1
		)
		self.assertEqual(
			humanized_dimensions.iloc[0]['dimension'], 'Point Temperature'
		)
		self.assertEqual(
			humanized_dimensions.iloc[0]['dimension_unity_code'], 1
		)
		self.assertEqual(
			humanized_dimensions.iloc[0]['dimension_unity'], '°C'
		)
		self.assertEqual(
			humanized_dimensions.iloc[0]['dimension_thing_code'], 15
		)
		self.assertEqual(
			humanized_dimensions.iloc[0]['dimension_thing'], 'Sensor_Inobram_T'
		)

		self.assertEqual(
			humanized_dimensions.iloc[1]['dimension_code'], 9
		)
		self.assertEqual(
			humanized_dimensions.iloc[1]['dimension'],
			'Point Water Temperature'
		)
		self.assertEqual(
			humanized_dimensions.iloc[1]['dimension_unity_code'], 1
		)
		self.assertEqual(
			humanized_dimensions.iloc[1]['dimension_unity'], '°C'
		)
		self.assertEqual(
			humanized_dimensions.iloc[1]['dimension_thing_code'], 15
		)
		self.assertEqual(
			humanized_dimensions.iloc[1]['dimension_thing'], 'Sensor_Inobram_T'
		)

	def test_profile(self):
		sensor = self.portia_api.device('2DPEQ572HEXP').port(4).sensor(1). \
			profile(params={
				'sort': True,
				'precision': 'ms',
				'timezone': 'Etc/UTC'
			})

		self.assertEqual(sensor.get('device'), '2DPEQ572HEXP')
		self.assertEqual(sensor.get('channel_id'), '1BTU7542G5EY')
		self.assertEqual(sensor.get('channel_code'), 14)
		self.assertEqual(sensor.get('thing_code'), 69)
		self.assertEqual(len(sensor.get('ports')), 1)
		self.assertEqual(len(sensor.get('ports')[0].get('sensors')), 1)

	def test_select(self):
		dimensions = self.portia_api.device('2DPEQ572HEXP').port(4) \
			.sensor(1).select(params={
		        'from': 1609470000000,
		        'to': 1609729199000,
		        'lower_bound': None,
		        'upper_bound': None,
		        'order': None,
		        'limit': None,
		        'precision': 'ms',
		        'timezone': 'Etc/UTC'
		    })

		for i, row in dimensions.iterrows():
			self.assertEqual(row.dimension_thing_code, 16)
			self.assertEqual(row.dimension_unity_code, 1)
			self.assertEqual(row.dimension_code, 1)

		for i, row in dimensions.humanize().iterrows():
			self.assertEqual(row.dimension_thing_code, 16)
			self.assertEqual(row.dimension_thing, 'Sensor_Inobram_TU')
			self.assertEqual(row.dimension_unity_code, 1)
			self.assertEqual(row.dimension_unity, '°C')
			self.assertEqual(row.dimension_code, 1)
			self.assertEqual(row.dimension, 'Point Temperature')

	def test_select_last(self):
		dimensions = self.portia_api.device('2DPEQ572HEXP').port(4) \
			.sensor(1).select(
				last=True,
				params={
			        'from': 1609470000000,
			        'to': 1609729199000,
			        'lower_bound': None,
			        'upper_bound': None,
			        'order': None,
			        'limit': None,
			        'precision': 'ms',
			        'timezone': 'Etc/UTC'
			    }
			)

		self.assertEqual(dimensions.iloc[0].dimension_thing_code, 16)
		self.assertEqual(dimensions.iloc[0].dimension_unity_code, 1)
		self.assertEqual(dimensions.iloc[0].dimension_code, 1)

		humanized_dimensions = dimensions.humanize()
		self.assertEqual(
			humanized_dimensions.iloc[0].dimension_thing_code, 16
		)
		self.assertEqual(
			humanized_dimensions.iloc[0].dimension_thing, 'Sensor_Inobram_TU'
		)
		self.assertEqual(
			humanized_dimensions.iloc[0].dimension_unity_code, 1
		)
		self.assertEqual(
			humanized_dimensions.iloc[0].dimension_unity, '°C'
		)
		self.assertEqual(
			humanized_dimensions.iloc[0].dimension_code, 1
		)
		self.assertEqual(
			humanized_dimensions.iloc[0].dimension, 'Point Temperature'
		)

	def test_summary(self):
		summarized_dimensions = self.portia_api.device('2DPEQ572HEXP') \
			.port(4).sensor(1).summary(params={
		        'from': 1609470000000,
		        'to': 1609729199000,
		        'lower_bound': None,
		        'upper_bound': None,
		        'offset': 0,
		        'fill': None,
		        'order': None,
		        'limit': None,
		        'avg': True,
		        'min': True,
		        'max': True,
		        'sum': False,
		        'median': False,
		        'mode': False,
		        'stddev': False,
		        'spread': False,
		        'last_timestamp': False,
		        'precision': 'ms',
		        'timezone': 'Etc/UTC'
		    })

		for i, row in summarized_dimensions.iterrows():
			self.assertEqual(row.dimension_unity_code, 1)

		for i, row in summarized_dimensions.humanize().iterrows():
			self.assertEqual(row.dimension_unity_code, 1)
			self.assertEqual(row.dimension_unity, '°C')

	def test_events(self):
		events = self.portia_api.device('2DPEQ572HEXP').port(4) \
			.sensor(1).events(params={
		        'from': 1609470000000,
		        'to': 1609729199000,
		        'order': None,
		        'limit': None,
		        'precision': 'ms',
		        'timezone': 'Etc/UTC'
		    })

		for i, row in events.iterrows():
			self.assertEqual(row.dimension_thing_code, 16)
			self.assertEqual(row.dimension_unity_code, 1)
			self.assertEqual(row.dimension_code, 1)
			self.assertEqual(row.event_code, 1)

		for i, row in events.humanize().iterrows():
			self.assertEqual(row.dimension_thing_code, 16)
			self.assertEqual(row.dimension_thing, 'Sensor_Inobram_TU')
			self.assertEqual(row.dimension_unity_code, 1)
			self.assertEqual(row.dimension_unity, '°C')
			self.assertEqual(row.dimension_code, 1)
			self.assertEqual(row.dimension, 'Point Temperature')
			self.assertEqual(row.event_code, 1)
			self.assertEqual(row.event, 'Communication State')

	def test_events_last(self):
		events = self.portia_api.device('2DPEQ572HEXP').port(4) \
			.sensor(1).events(
				last=True,
				params={
			        'from': 1609470000000,
			        'to': 1609729199000,
			        'order': None,
			        'limit': None,
			        'precision': 'ms',
			        'timezone': 'Etc/UTC'
			    }
			)

		self.assertEqual(events.iloc[0].dimension_thing_code, 16)
		self.assertEqual(events.iloc[0].dimension_unity_code, 1)
		self.assertEqual(events.iloc[0].dimension_code, 1)
		self.assertEqual(events.iloc[0].event_code, 1)

		humanized_events = events.humanize()
		self.assertEqual(
			humanized_events.iloc[0].dimension_thing_code, 16
		)
		self.assertEqual(
			humanized_events.iloc[0].dimension_thing, 'Sensor_Inobram_TU'
		)
		self.assertEqual(
			humanized_events.iloc[0].dimension_unity_code, 1
		)
		self.assertEqual(
			humanized_events.iloc[0].dimension_unity, '°C'
		)
		self.assertEqual(
			humanized_events.iloc[0].dimension_code, 1
		)
		self.assertEqual(
			humanized_events.iloc[0].dimension, 'Point Temperature'
		)
		self.assertEqual(
			humanized_events.iloc[0].event_code, 1
		)
		self.assertEqual(
			humanized_events.iloc[0].event, 'Communication State'
		)


class TestEdgeDeviceDimensionFromDevice(unittest.TestCase):
	"""Set of integration tests for all functions concerning the
	EdgeDeviceDimensionFromDevice instance.
	"""
	@classmethod
	def setUpClass(cls):
		"""Creates a Portia configuration to test the instance.
		"""

		# Settings for test
		base_url = os.getenv('PORTIA_URL', 'https://api-portia.agriness.io/v3')
		token = os.getenv('PORTIA_TOKEN')

		# Creating Portia API instance
		cls.portia_api = portia.PortiaApi({
			'baseurl': base_url,
			'authorization': token,
			'debug': False,
			'Accept': 'text/csv'
		})

	def test_summary(self):
		summarized_dimensions = self.portia_api.device('2DPEQ572HEXP') \
			.dimension(1).summary(None, params={
		        'from': 1609470000000,
		        'to': 1609729199000,
		        'lower_bound': None,
		        'upper_bound': None,
		        'offset': 0,
		        'fill': None,
		        'order': None,
		        'limit': None,
		        'avg': True,
		        'min': True,
		        'max': True,
		        'sum': False,
		        'median': False,
		        'mode': False,
		        'stddev': False,
		        'spread': False,
		        'last_timestamp': False,
		        'precision': 'ms',
		        'timezone': 'Etc/UTC'
		    })

		for i, row in summarized_dimensions.iterrows():
			self.assertEqual(row.dimension_unity_code, 1)

		for i, row in summarized_dimensions.humanize().iterrows():
			self.assertEqual(row.dimension_unity_code, 1)
			self.assertEqual(row.dimension_unity, '°C')



class TestEdgeDeviceDimensionFromPort(unittest.TestCase):
	"""Set of integration tests for all functions concerning the
	EdgeDeviceDimensionFromPort instance.
	"""
	@classmethod
	def setUpClass(cls):
		"""Creates a Portia configuration to test the instance.
		"""

		# Settings for test
		base_url = os.getenv('PORTIA_URL', 'https://api-portia.agriness.io/v3')
		token = os.getenv('PORTIA_TOKEN')

		# Creating Portia API instance
		cls.portia_api = portia.PortiaApi({
			'baseurl': base_url,
			'authorization': token,
			'debug': False,
			'Accept': 'text/csv'
		})

	def test_select(self):
		dimensions = self.portia_api.device('2DPEQ572HEXP').port(4) \
			.dimension(1).select(params={
		        'from': 1609470000000,
		        'to': 1609729199000,
		        'lower_bound': None,
		        'upper_bound': None,
		        'order': None,
		        'limit': None,
		        'precision': 'ms',
		        'timezone': 'Etc/UTC'
		    })

		for i, row in dimensions.iterrows():
			self.assertEqual(row.dimension_thing_code, 16)
			self.assertEqual(row.dimension_unity_code, 1)
			self.assertEqual(row.dimension_code, 1)

		for i, row in dimensions.humanize().iterrows():
			self.assertEqual(row.dimension_thing_code, 16)
			self.assertEqual(row.dimension_thing, 'Sensor_Inobram_TU')
			self.assertEqual(row.dimension_unity_code, 1)
			self.assertEqual(row.dimension_unity, '°C')
			self.assertEqual(row.dimension_code, 1)
			self.assertEqual(row.dimension, 'Point Temperature')

	def test_select_last(self):
		dimensions = self.portia_api.device('2DPEQ572HEXP').port(4) \
			.dimension(1).select(
				last=True,
				params={
			        'from': 1609470000000,
			        'to': 1609729199000,
			        'lower_bound': None,
			        'upper_bound': None,
			        'order': None,
			        'limit': None,
			        'precision': 'ms',
			        'timezone': 'Etc/UTC'
			    }
			)

		self.assertEqual(dimensions.iloc[0].dimension_thing_code, 16)
		self.assertEqual(dimensions.iloc[0].dimension_unity_code, 1)
		self.assertEqual(dimensions.iloc[0].dimension_code, 1)

		humanized_dimensions = dimensions.humanize()
		self.assertEqual(
			humanized_dimensions.iloc[0].dimension_thing_code, 16
		)
		self.assertEqual(
			humanized_dimensions.iloc[0].dimension_thing, 'Sensor_Inobram_TU'
		)
		self.assertEqual(
			humanized_dimensions.iloc[0].dimension_unity_code, 1
		)
		self.assertEqual(
			humanized_dimensions.iloc[0].dimension_unity, '°C'
		)
		self.assertEqual(
			humanized_dimensions.iloc[0].dimension_code, 1
		)
		self.assertEqual(
			humanized_dimensions.iloc[0].dimension, 'Point Temperature'
		)

	def test_events(self):
		events = self.portia_api.device('2DPEQ572HEXP').port(4) \
			.dimension(1).events(params={
		        'from': 1609470000000,
		        'to': 1609729199000,
		        'order': None,
		        'limit': None,
		        'precision': 'ms',
		        'timezone': 'Etc/UTC'
		    })

		for i, row in events.iterrows():
			self.assertEqual(row.dimension_thing_code, 16)
			self.assertEqual(row.dimension_unity_code, 1)
			self.assertEqual(row.dimension_code, 1)
			self.assertEqual(row.event_code, 1)

		for i, row in events.humanize().iterrows():
			self.assertEqual(row.dimension_thing_code, 16)
			self.assertEqual(row.dimension_thing, 'Sensor_Inobram_TU')
			self.assertEqual(row.dimension_unity_code, 1)
			self.assertEqual(row.dimension_unity, '°C')
			self.assertEqual(row.dimension_code, 1)
			self.assertEqual(row.dimension, 'Point Temperature')
			self.assertEqual(row.event_code, 1)
			self.assertEqual(row.event, 'Communication State')

	def test_events_last(self):
		events = self.portia_api.device('2DPEQ572HEXP').port(4) \
			.dimension(1).events(
				last=True,
				params={
			        'from': 1609470000000,
			        'to': 1609729199000,
			        'order': None,
			        'limit': None,
			        'precision': 'ms',
			        'timezone': 'Etc/UTC'
			    }
			)

		self.assertEqual(events.iloc[0].dimension_thing_code, 16)
		self.assertEqual(events.iloc[0].dimension_unity_code, 1)
		self.assertEqual(events.iloc[0].dimension_code, 1)
		self.assertEqual(events.iloc[0].event_code, 1)

		humanized_events = events.humanize()
		self.assertEqual(
			humanized_events.iloc[0].dimension_thing_code, 16
		)
		self.assertEqual(
			humanized_events.iloc[0].dimension_thing, 'Sensor_Inobram_TU'
		)
		self.assertEqual(
			humanized_events.iloc[0].dimension_unity_code, 1
		)
		self.assertEqual(
			humanized_events.iloc[0].dimension_unity, '°C'
		)
		self.assertEqual(
			humanized_events.iloc[0].dimension_code, 1
		)
		self.assertEqual(
			humanized_events.iloc[0].dimension, 'Point Temperature'
		)
		self.assertEqual(
			humanized_events.iloc[0].event_code, 1
		)
		self.assertEqual(
			humanized_events.iloc[0].event, 'Communication State'
		)


class TestEdgeDeviceDimensionFromSensor(unittest.TestCase):
	"""Set of integration tests for all functions concerning the
	EdgeDeviceDimensionFromSensor instance.
	"""
	@classmethod
	def setUpClass(cls):
		"""Creates a Portia configuration to test the instance.
		"""

		# Settings for test
		base_url = os.getenv('PORTIA_URL', 'https://api-portia.agriness.io/v3')
		token = os.getenv('PORTIA_TOKEN')

		# Creating Portia API instance
		cls.portia_api = portia.PortiaApi({
			'baseurl': base_url,
			'authorization': token,
			'debug': False,
			'Accept': 'text/csv'
		})

	def test_select(self):
		dimensions = self.portia_api.device('2DPEQ572HEXP').port(4).sensor(1) \
			.dimension(1).select(params={
		        'from': 1609470000000,
		        'to': 1609729199000,
		        'lower_bound': None,
		        'upper_bound': None,
		        'order': None,
		        'limit': None,
		        'precision': 'ms',
		        'timezone': 'Etc/UTC'
		    })

		for i, row in dimensions.iterrows():
			self.assertEqual(row.dimension_thing_code, 16)
			self.assertEqual(row.dimension_unity_code, 1)
			self.assertEqual(row.dimension_code, 1)

		for i, row in dimensions.humanize().iterrows():
			self.assertEqual(row.dimension_thing_code, 16)
			self.assertEqual(row.dimension_thing, 'Sensor_Inobram_TU')
			self.assertEqual(row.dimension_unity_code, 1)
			self.assertEqual(row.dimension_unity, '°C')
			self.assertEqual(row.dimension_code, 1)
			self.assertEqual(row.dimension, 'Point Temperature')

	def test_select_last(self):
		dimensions = self.portia_api.device('2DPEQ572HEXP').port(4).sensor(1) \
			.dimension(1).select(
				last=True,
				params={
			        'from': 1609470000000,
			        'to': 1609729199000,
			        'lower_bound': None,
			        'upper_bound': None,
			        'order': None,
			        'limit': None,
			        'precision': 'ms',
			        'timezone': 'Etc/UTC'
			    }
			)

		self.assertEqual(dimensions.iloc[0].dimension_thing_code, 16)
		self.assertEqual(dimensions.iloc[0].dimension_unity_code, 1)
		self.assertEqual(dimensions.iloc[0].dimension_code, 1)

		humanized_dimensions = dimensions.humanize()
		self.assertEqual(
			humanized_dimensions.iloc[0].dimension_thing_code, 16
		)
		self.assertEqual(
			humanized_dimensions.iloc[0].dimension_thing, 'Sensor_Inobram_TU'
		)
		self.assertEqual(
			humanized_dimensions.iloc[0].dimension_unity_code, 1
		)
		self.assertEqual(
			humanized_dimensions.iloc[0].dimension_unity, '°C'
		)
		self.assertEqual(
			humanized_dimensions.iloc[0].dimension_code, 1
		)
		self.assertEqual(
			humanized_dimensions.iloc[0].dimension, 'Point Temperature'
		)

	def test_summary(self):
		summarized_dimensions = self.portia_api.device('2DPEQ572HEXP') \
			.port(4).sensor(1).dimension(1).summary(params={
		        'from': 1609470000000,
		        'to': 1609729199000,
		        'lower_bound': None,
		        'upper_bound': None,
		        'offset': 0,
		        'fill': None,
		        'order': None,
		        'limit': None,
		        'avg': True,
		        'min': True,
		        'max': True,
		        'sum': False,
		        'median': False,
		        'mode': False,
		        'stddev': False,
		        'spread': False,
		        'last_timestamp': False,
		        'precision': 'ms',
		        'timezone': 'Etc/UTC'
		    })

		for i, row in summarized_dimensions.iterrows():
			self.assertEqual(row.dimension_unity_code, 1)

		for i, row in summarized_dimensions.humanize().iterrows():
			self.assertEqual(row.dimension_unity_code, 1)
			self.assertEqual(row.dimension_unity, '°C')

	def test_events(self):
		events = self.portia_api.device('2DPEQ572HEXP').port(4).sensor(1) \
			.dimension(1).events(params={
		        'from': 1609470000000,
		        'to': 1609729199000,
		        'order': None,
		        'limit': None,
		        'precision': 'ms',
		        'timezone': 'Etc/UTC'
		    })

		for i, row in events.iterrows():
			self.assertEqual(row.dimension_thing_code, 16)
			self.assertEqual(row.dimension_unity_code, 1)
			self.assertEqual(row.dimension_code, 1)
			self.assertEqual(row.event_code, 1)

		for i, row in events.humanize().iterrows():
			self.assertEqual(row.dimension_thing_code, 16)
			self.assertEqual(row.dimension_thing, 'Sensor_Inobram_TU')
			self.assertEqual(row.dimension_unity_code, 1)
			self.assertEqual(row.dimension_unity, '°C')
			self.assertEqual(row.dimension_code, 1)
			self.assertEqual(row.dimension, 'Point Temperature')
			self.assertEqual(row.event_code, 1)
			self.assertEqual(row.event, 'Communication State')

	def test_events_last(self):
		events = self.portia_api.device('2DPEQ572HEXP').port(4).sensor(1) \
			.dimension(1).events(
				last=True,
				params={
			        'from': 1609470000000,
			        'to': 1609729199000,
			        'order': None,
			        'limit': None,
			        'precision': 'ms',
			        'timezone': 'Etc/UTC'
			    }
			)

		self.assertEqual(events.iloc[0].dimension_thing_code, 16)
		self.assertEqual(events.iloc[0].dimension_unity_code, 1)
		self.assertEqual(events.iloc[0].dimension_code, 1)
		self.assertEqual(events.iloc[0].event_code, 1)

		humanized_events = events.humanize()
		self.assertEqual(
			humanized_events.iloc[0].dimension_thing_code, 16
		)
		self.assertEqual(
			humanized_events.iloc[0].dimension_thing, 'Sensor_Inobram_TU'
		)
		self.assertEqual(
			humanized_events.iloc[0].dimension_unity_code, 1
		)
		self.assertEqual(
			humanized_events.iloc[0].dimension_unity, '°C'
		)
		self.assertEqual(
			humanized_events.iloc[0].dimension_code, 1
		)
		self.assertEqual(
			humanized_events.iloc[0].dimension, 'Point Temperature'
		)
		self.assertEqual(
			humanized_events.iloc[0].event_code, 1
		)
		self.assertEqual(
			humanized_events.iloc[0].event, 'Communication State'
		)


class TestEdgeDeviceEventFromSensor(unittest.TestCase):
	"""Set of integration tests for all functions concerning the
	EdgeDeviceEventFromSensor instance.
	"""
	@classmethod
	def setUpClass(cls):
		"""Creates a Portia configuration to test the instance.
		"""

		# Settings for test
		base_url = os.getenv('PORTIA_URL', 'https://api-portia.agriness.io/v3')
		token = os.getenv('PORTIA_TOKEN')

		# Creating Portia API instance
		cls.portia_api = portia.PortiaApi({
			'baseurl': base_url,
			'authorization': token,
			'debug': False,
			'Accept': 'text/csv'
		})

	def test_events(self):
		events = self.portia_api.device('2DPEQ572HEXP').port(4).sensor(1) \
			.event(1).events(params={
		        'from': 1609470000000,
		        'to': 1609729199000,
		        'order': None,
		        'limit': None,
		        'precision': 'ms',
		        'timezone': 'Etc/UTC'
		    })

		for i, row in events.iterrows():
			self.assertEqual(row.dimension_thing_code, 16)
			self.assertEqual(row.dimension_unity_code, 1)
			self.assertEqual(row.dimension_code, 1)
			self.assertEqual(row.event_code, 1)

		for i, row in events.humanize().iterrows():
			self.assertEqual(row.dimension_thing_code, 16)
			self.assertEqual(row.dimension_thing, 'Sensor_Inobram_TU')
			self.assertEqual(row.dimension_unity_code, 1)
			self.assertEqual(row.dimension_unity, '°C')
			self.assertEqual(row.dimension_code, 1)
			self.assertEqual(row.dimension, 'Point Temperature')
			self.assertEqual(row.event_code, 1)
			self.assertEqual(row.event, 'Communication State')

	def test_events_last(self):
		events = self.portia_api.device('2DPEQ572HEXP').port(4).sensor(1) \
			.event(1).events(
				last=True,
				params={
			        'from': 1609470000000,
			        'to': 1609729199000,
			        'order': None,
			        'limit': None,
			        'precision': 'ms',
			        'timezone': 'Etc/UTC'
			    }
			)

		self.assertEqual(events.iloc[0].dimension_thing_code, 16)
		self.assertEqual(events.iloc[0].dimension_unity_code, 1)
		self.assertEqual(events.iloc[0].dimension_code, 1)
		self.assertEqual(events.iloc[0].event_code, 1)

		humanized_events = events.humanize()
		self.assertEqual(
			humanized_events.iloc[0].dimension_thing_code, 16
		)
		self.assertEqual(
			humanized_events.iloc[0].dimension_thing, 'Sensor_Inobram_TU'
		)
		self.assertEqual(
			humanized_events.iloc[0].dimension_unity_code, 1
		)
		self.assertEqual(
			humanized_events.iloc[0].dimension_unity, '°C'
		)
		self.assertEqual(
			humanized_events.iloc[0].dimension_code, 1
		)
		self.assertEqual(
			humanized_events.iloc[0].dimension, 'Point Temperature'
		)
		self.assertEqual(
			humanized_events.iloc[0].event_code, 1
		)
		self.assertEqual(
			humanized_events.iloc[0].event, 'Communication State'
		)


class TestEdgeDeviceEventFromDimension(unittest.TestCase):
	"""Set of integration tests for all functions concerning the
	EdgeDeviceEventFromDimension instance.
	"""
	@classmethod
	def setUpClass(cls):
		"""Creates a Portia configuration to test the instance.
		"""

		# Settings for test
		base_url = os.getenv('PORTIA_URL', 'https://api-portia.agriness.io/v3')
		token = os.getenv('PORTIA_TOKEN')

		# Creating Portia API instance
		cls.portia_api = portia.PortiaApi({
			'baseurl': base_url,
			'authorization': token,
			'debug': False,
			'Accept': 'text/csv'
		})

	def test_events(self):
		events = self.portia_api.device('2DPEQ572HEXP').port(4).dimension(1) \
			.event(1).events(params={
		        'from': 1609470000000,
		        'to': 1609729199000,
		        'order': None,
		        'limit': None,
		        'precision': 'ms',
		        'timezone': 'Etc/UTC'
		    })

		for i, row in events.iterrows():
			self.assertEqual(row.dimension_thing_code, 16)
			self.assertEqual(row.dimension_unity_code, 1)
			self.assertEqual(row.dimension_code, 1)
			self.assertEqual(row.event_code, 1)

		for i, row in events.humanize().iterrows():
			self.assertEqual(row.dimension_thing_code, 16)
			self.assertEqual(row.dimension_thing, 'Sensor_Inobram_TU')
			self.assertEqual(row.dimension_unity_code, 1)
			self.assertEqual(row.dimension_unity, '°C')
			self.assertEqual(row.dimension_code, 1)
			self.assertEqual(row.dimension, 'Point Temperature')
			self.assertEqual(row.event_code, 1)
			self.assertEqual(row.event, 'Communication State')

	def test_events_last(self):
		events = self.portia_api.device('2DPEQ572HEXP').port(4).dimension(1) \
			.event(1).events(
				last=True,
				params={
			        'from': 1609470000000,
			        'to': 1609729199000,
			        'order': None,
			        'limit': None,
			        'precision': 'ms',
			        'timezone': 'Etc/UTC'
			    }
			)

		self.assertEqual(events.iloc[0].dimension_thing_code, 16)
		self.assertEqual(events.iloc[0].dimension_unity_code, 1)
		self.assertEqual(events.iloc[0].dimension_code, 1)
		self.assertEqual(events.iloc[0].event_code, 1)

		humanized_events = events.humanize()
		self.assertEqual(
			humanized_events.iloc[0].dimension_thing_code, 16
		)
		self.assertEqual(
			humanized_events.iloc[0].dimension_thing, 'Sensor_Inobram_TU'
		)
		self.assertEqual(
			humanized_events.iloc[0].dimension_unity_code, 1
		)
		self.assertEqual(
			humanized_events.iloc[0].dimension_unity, '°C'
		)
		self.assertEqual(
			humanized_events.iloc[0].dimension_code, 1
		)
		self.assertEqual(
			humanized_events.iloc[0].dimension, 'Point Temperature'
		)
		self.assertEqual(
			humanized_events.iloc[0].event_code, 1
		)
		self.assertEqual(
			humanized_events.iloc[0].event, 'Communication State'
		)


class TestEdgeDeviceEventFromSensorDimension(unittest.TestCase):
	"""Set of integration tests for all functions concerning the
	EdgeDeviceEventFromSensorDimension instance.
	"""
	@classmethod
	def setUpClass(cls):
		"""Creates a Portia configuration to test the instance.
		"""

		# Settings for test
		base_url = os.getenv('PORTIA_URL', 'https://api-portia.agriness.io/v3')
		token = os.getenv('PORTIA_TOKEN')

		# Creating Portia API instance
		cls.portia_api = portia.PortiaApi({
			'baseurl': base_url,
			'authorization': token,
			'debug': False,
			'Accept': 'text/csv'
		})

	def test_events(self):
		events = self.portia_api.device('2DPEQ572HEXP').port(4).sensor(1) \
			.dimension(1).event(1).events(params={
		        'from': 1609470000000,
		        'to': 1609729199000,
		        'order': None,
		        'limit': None,
		        'precision': 'ms',
		        'timezone': 'Etc/UTC'
		    })

		for i, row in events.iterrows():
			self.assertEqual(row.dimension_thing_code, 16)
			self.assertEqual(row.dimension_unity_code, 1)
			self.assertEqual(row.dimension_code, 1)
			self.assertEqual(row.event_code, 1)

		for i, row in events.humanize().iterrows():
			self.assertEqual(row.dimension_thing_code, 16)
			self.assertEqual(row.dimension_thing, 'Sensor_Inobram_TU')
			self.assertEqual(row.dimension_unity_code, 1)
			self.assertEqual(row.dimension_unity, '°C')
			self.assertEqual(row.dimension_code, 1)
			self.assertEqual(row.dimension, 'Point Temperature')
			self.assertEqual(row.event_code, 1)
			self.assertEqual(row.event, 'Communication State')

	def test_events_last(self):
		events = self.portia_api.device('2DPEQ572HEXP').port(4).sensor(1) \
			.dimension(1).event(1).events(
				last=True,
				params={
			        'from': 1609470000000,
			        'to': 1609729199000,
			        'order': None,
			        'limit': None,
			        'precision': 'ms',
			        'timezone': 'Etc/UTC'
			    }
			)

		self.assertEqual(events.iloc[0].dimension_thing_code, 16)
		self.assertEqual(events.iloc[0].dimension_unity_code, 1)
		self.assertEqual(events.iloc[0].dimension_code, 1)
		self.assertEqual(events.iloc[0].event_code, 1)

		humanized_events = events.humanize()
		self.assertEqual(
			humanized_events.iloc[0].dimension_thing_code, 16
		)
		self.assertEqual(
			humanized_events.iloc[0].dimension_thing, 'Sensor_Inobram_TU'
		)
		self.assertEqual(
			humanized_events.iloc[0].dimension_unity_code, 1
		)
		self.assertEqual(
			humanized_events.iloc[0].dimension_unity, '°C'
		)
		self.assertEqual(
			humanized_events.iloc[0].dimension_code, 1
		)
		self.assertEqual(
			humanized_events.iloc[0].dimension, 'Point Temperature'
		)
		self.assertEqual(
			humanized_events.iloc[0].event_code, 1
		)
		self.assertEqual(
			humanized_events.iloc[0].event, 'Communication State'
		)


if __name__ == '__main__':
	unittest.main()



#!/usr/bin/env python
from os.path import join

from setuptools import setup, find_packages


MODULE_NAME = 'portiapy'
REPO_NAME = 'portiapy'


with open('./README.md') as f:
    readme = f.read()


setup(
    name=MODULE_NAME,
    description='A small package for handling Agriness Edge\'s REST API',
    long_description=readme,
    long_description_content_type='text/markdown',
    url='https://github.com/AgrinessEdgeIoT/{:s}'.format(REPO_NAME),
    author='Matheus Mota, Lucas Góes',
    author_email='matheus@agrinessedge.com, lucas@agrinessedge.com',
    packages=find_packages(exclude=('binder', 'tests', 'docs')),
    version=open(join(MODULE_NAME, 'VERSION')).read().strip(),
    install_requires=['requests>=2.20.0', 'pytz>=2018.5',
                      'python-dateutil>=2.7.3', 'plotly>=3.1.1',
                      'pandas>=0.23.4', 'arrow>=0.12.1'],
    classifiers=(
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ),
    include_package_data=True
)



"""A class to abstract the usage of all endpoints.
"""

import types
import builtins

import pandas as pd

import portiapy.specs as specs
import portiapy.utils as utils
import portiapy.axioms as axioms
import portiapy.events as events
import portiapy.phases as phases
import portiapy.select as select
import portiapy.profile as profile
import portiapy.summary as summary
import portiapy.describe as describe


class CustomDict(dict):
	"""Custom dictionary with an humanization method.
	
	Extends:
		dict
	"""
	def humanize(
		self,
		locale='en-us',
		custom_dimension: dict=None,
		custom_event: dict=None,
		custom_unity: dict=None
	) -> dict:
		"""Humanizes dictionary.
		
		Keyword Arguments:
	        locale {str} -- which language to use when humanizing
	                        (default {'en-us'})
	        custom_dimension {dict} -- custom list of dimension codes
	                                   (default {None})
	        custom_event {dict} -- custom list of event codes (default {None})
	        custom_unity {dict} -- custom list of unity codes (default {None})
		
		Returns:
			dict -- humanized dictionary
		"""
		return utils.humanize_json(
			self, locale, custom_dimension, custom_unity
		)


def add_humanize_method(obj: object) -> object:
	"""Adds humanize method to an object's instance.
	
	Arguments:
		obj {object} -- object to have the method added
	
	Returns:
		object -- object with the humanize method
	"""
	if isinstance(obj, pd.DataFrame):
		obj.humanize = types.MethodType(utils.humanize_dataframe, obj)
	elif isinstance(obj, dict):
		obj = CustomDict(obj)

	return obj


class PortiaApi(object):
	"""A factory for devices that use our Portia API.
	"""
	def __init__(self, portia_config: dict):
		"""PortiaApi's constructor.
		
		Arguments:
			portia_config {dict} -- Portia's configuration arguments
		"""
		self.portia_config = portia_config

	def pipeline(self) -> 'EdgePipeline':
		"""Builds a new EdgePipeline instance.
		
		Returns:
			EdgePipeline -- EdgePipeline instance
		"""
		return EdgePipeline(self.portia_config)

	def device(self, edge_id: str) -> 'EdgeDevice':
		"""Builds a new EdgeDevice instance.
		
		Arguments:
			edge_id {str} -- Edge ID that identifies the device

		Returns:
			EdgeDevice -- EdgeDevice instance
		"""
		return EdgeDevice(edge_id, self.portia_config)


class EdgePipeline(object):
	"""Abstracts usage of pipeline endpoints.
	"""
	def __init__(self, portia_config: dict):
		"""EdgePipeline's constructor.
		
		Arguments:
			portia_config {dict} -- Portia's configuration arguments
		"""
		self.portia_config = portia_config

	def phase(self) -> 'EdgePipelinePhase':
		"""Builds a new EdgePipelinePhase instance.
		
		Returns:
			EdgePipelinePhase -- EdgePipelinePhase instance
		"""
		return EdgePipelinePhase(self.portia_config)

	def axiom(self) -> 'EdgePipelineAxiom':
		"""Builds a new EdgePipelineAxiom instance.
		
		Returns:
			EdgePipelineAxiom -- EdgePipelineAxiom instance
		"""
		return EdgePipelineAxiom(self.portia_config)

	def specification(self) -> 'EdgePipelineSpecification':
		"""Builds a new EdgePipelineSpecification instance.
		
		Returns:
			EdgePipelineSpecification -- EdgePipelineSpecification instance
		"""
		return EdgePipelineSpecification(self.portia_config)


class EdgePipelinePhase(object):
	"""Abstracts usage of pipeline phase endpoints.
	"""
	def __init__(self, portia_config: dict):
		"""EdgePipelinePhase's constructor.
		
		Arguments:
			portia_config {dict} -- Portia's configuration arguments
		"""
		self.portia_config = portia_config

	def list(self) -> list:
		return phases.index(self.portia_config)

	def create(self, payload):
		return phases.store(self.portia_config, payload)

	def display(self, phase_name):
		return phases.show(self.portia_config, phase_name)

	def update(self, phase_name, payload):
		return phases.update(self.portia_config, phase_name, payload)


class EdgePipelineAxiom(object):
	"""Abstracts usage of pipeline axiom endpoints.
	"""
	def __init__(self, portia_config: dict):
		"""EdgePipelineAxiom's constructor.
		
		Arguments:
			portia_config {dict} -- Portia's configuration arguments
		"""
		self.portia_config = portia_config

	def list(self):
		return axioms.index(self.portia_config)

	def create(self, payload):
		return axioms.store(self.portia_config, payload)

	def display(self, axiom_name, show_params=False):
		return axioms.show(self.portia_config, axiom_name, show_params)

	def update(self, axiom_name, payload):
		return axioms.update(self.portia_config, axiom_name, payload)

	def delete(self, axiom_name):
		return axioms.destroy(self.portia_config, axiom_name)


class EdgePipelineSpecification(object):
	"""Abstracts usage of pipeline specification endpoints.
	"""
	def __init__(self, portia_config: dict):
		"""EdgePipelineSpecification's constructor.
		
		Arguments:
			portia_config {dict} -- Portia's configuration arguments
		"""
		self.portia_config = portia_config

	def list(self):
		return specs.index(self.portia_config)

	def create(self, payload):
		return specs.store(self.portia_config, payload)

	def display(self, specName):
		return specs.show(self.portia_config, specName)

	def update(self, specName, payload):
		return specs.update(self.portia_config, specName, payload)

	def delete(self, specName):
		return specs.destroy(self.portia_config, specName)


class EdgeDevice(object):
	"""Abstracts usage of all Portia endpoints concerning data that only need
	an Edge ID.
	"""
	def __init__(self, edge_id: str, portia_config: dict):
		"""EdgeDevice's constructor.
		
		Arguments:
			edge_id {str} -- Edge ID that identifies the device
			portia_config {dict} -- Portia's configuration arguments
		"""
		self.edge_id = edge_id
		self.portia_config = portia_config

	def port(self, port: int) -> 'EdgeDevicePort':
		"""Builds a new EdgeDevicePort instance.
		
		Arguments:
			port {int} -- port of the device

		Returns:
			EdgeDevicePort -- EdgeDevicePort instance
		"""
		return EdgeDevicePort(self, port, self.portia_config)

	def dimension(self, dimension: int) -> 'EdgeDeviceDimensionFromDevice':
		"""Builds a new EdgeDeviceDimensionFromDevice instance.
		
		Arguments:
			dimension {int} -- dimension of the device

		Returns:
			EdgeDeviceDimensionFromDevice -- EdgeDeviceDimensionFromDevice
											 instance
		"""
		return EdgeDeviceDimensionFromDevice(
			self, dimension, self.portia_config
		)

	def ports(self, last: bool=False, params: dict=None) -> object:
		"""Lists a device's ports.
		
	    Keyword Arguments:
	        last {bool} -- if the last package of each port should be returned
	        			   or not (default: {False})
	        params {dict} -- params to send to the service (default: {None})

	    Returns:
	        object -- object with the list of ports
		"""
		return add_humanize_method(describe.device_ports(
			self.portia_config, self.edge_id, last, params
		))

	def profile(
		self,
		strategy: 'ProfileStrategies'=profile.ProfileStrategies.BY_ZERO_PORT,
		interval: int=30,
		params: dict=None
	) -> dict:
		"""Retrieves a device's profile.

	    Keyword Arguments:
	        strategy {ProfileStrategies} -- strategy to use when building the
	                                        profile (default: 
	                                        {ProfileStrategies.BY_ZERO_PORT})
	        interval {int} -- interval of time in minutes to build the profile
	                          (default: {30})
	        params {dict} -- params to send to the service (default: {None})
		
	    Returns:
	        dict -- dictionary with the device's profile
		"""
		return add_humanize_method(profile.device_profile(
			self.portia_config, self.edge_id, strategy, interval, params
		))


class EdgeDevicePort(object):
	"""Abstracts usage of all Portia endpoints concerning data that only need
	an Edge ID and a port.
	"""
	def __init__(
		self, edge_device: EdgeDevice, port: int, portia_config: dict
	):
		"""EdgeDevicePort's constructor.
		
		Arguments:
			edge_device {EdgeDevice} -- instance of an Edge device
			port {int} -- port of the device
			portia_config {dict} -- Portia's configuration arguments
		"""
		self.edge_id = edge_device.edge_id
		self.port = port
		self.portia_config = portia_config

	def sensor(self, sensor: int) -> 'EdgeDeviceSensor':
		"""Builds a new EdgeDeviceSensor instance.
		
		Arguments:
			sensor {int} -- sensor of the device

		Returns:
			EdgeDeviceSensor -- EdgeDeviceSensor instance
		"""
		return EdgeDeviceSensor(self, sensor, self.portia_config)

	def dimension(self, dimension: int) -> 'EdgeDeviceDimensionFromPort':
		"""Builds a new EdgeDeviceDimensionFromPort instance.
		
		Arguments:
			dimension {int} -- dimension code of the device

		Returns:
			EdgeDeviceDimensionFromPort -- EdgeDeviceDimensionFromPort instance
		"""
		return EdgeDeviceDimensionFromPort(self, dimension, self.portia_config)

	def sensors(self, last: bool=False, params: dict=None) -> object:
		"""Lists a device's sensors.
		
	    Keyword Arguments:
	        last {bool} -- if the last package of each sensor should be
	        			   returned or not (default: {False})
	        params {dict} -- params to send to the service (default: {None})

	    Returns:
	        object -- object with the list of sensors
		"""
		return add_humanize_method(describe.device_port_sensors(
			self.portia_config, self.edge_id, self.port, last, params
		))

	def dimensions(self, last: bool=False, params: dict=None) -> object:
		"""Lists a device's dimensions.
		
	    Keyword Arguments:
	        last {bool} -- if the last package of each dimension should be
	        			   returned or not (default: {False})
	        params {dict} -- params to send to the service (default: {None})

	    Returns:
	        object -- object with the list of dimensions
		"""
		return add_humanize_method(describe.device_port_dimensions(
			self.portia_config, self.edge_id, self.port, last, params
		))

	def profile(
		self,
		strategy: 'ProfileStrategies'=profile.ProfileStrategies.BY_ZERO_PORT,
		interval: int=30,
		params: dict=None
	) -> dict:
		"""Retrieves a port's profile.

	    Keyword Arguments:
	        strategy {ProfileStrategies} -- strategy to use when building the
	                                        profile (default: 
	                                        {ProfileStrategies.BY_ZERO_PORT})
	        interval {int} -- interval of time in minutes to build the profile
	                          (default: {30})
	        params {dict} -- params to send to the service (default: {None})
		
	    Returns:
	        dict -- dictionary with the port's profile
		"""
		return add_humanize_method(profile.port_profile(
			self.portia_config,
			self.edge_id,
			self.port,
			strategy,
			interval,
			params
		))


class EdgeDeviceSensor(object):
	"""Abstracts usage of all Portia endpoints concerning data that only need
	an Edge ID, a port and a sensor.
	"""
	def __init__(
		self,
		edge_device_port: EdgeDevicePort,
		sensor: int,
		portia_config: dict
	):
		"""EdgeDeviceSensor's constructor.
		
		Arguments:
			edge_device_port {EdgeDevicePort} -- instance of an Edge device
												 port
			sensor {int} -- sensor of the device
			portia_config {dict} -- Portia's configuration arguments
		"""
		self.edge_id = edge_device_port.edge_id
		self.port = edge_device_port.port
		self.sensor = sensor
		self.portia_config = portia_config

	def dimension(self, dimension: int) -> 'EdgeDeviceDimensionFromSensor':
		"""Builds a new EdgeDeviceDimensionFromSensor instance.
		
		Arguments:
			dimension {int} -- dimension code of the device

		Returns:
			EdgeDeviceDimensionFromSensor -- EdgeDeviceDimensionFromSensor
											 instance
		"""
		return EdgeDeviceDimensionFromSensor(
			self, dimension, self.portia_config
		)

	def event(self, event: int) -> 'EdgeDeviceEventFromSensor':
		"""Builds a new EdgeDeviceEventFromSensor instance.
		
		Arguments:
			event {int} -- event code of the device

		Returns:
			EdgeDeviceEventFromSensor -- EdgeDeviceEventFromSensor instance
		"""
		return EdgeDeviceEventFromSensor(self, event, self.portia_config)

	def dimensions(self, last: bool=False, params: dict=None) -> object:
		"""Lists a device's dimensions.
		
	    Keyword Arguments:
	        last {bool} -- if the last package of each dimension should be
	        			   returned or not (default: {False})
	        params {dict} -- params to send to the service (default: {None})

	    Returns:
	        object -- object with the list of dimensions
		"""
		return add_humanize_method(describe.device_port_sensor_dimensions(
			self.portia_config,
			self.edge_id,
			self.port,
			self.sensor,
			last,
			params
		))

	def profile(
		self,
		strategy: 'ProfileStrategies'=profile.ProfileStrategies.BY_ZERO_PORT,
		interval: int=30,
		params: dict=None
	) -> dict:
		"""Retrieves a sensor's profile.

	    Keyword Arguments:
	        strategy {ProfileStrategies} -- strategy to use when building the
	                                        profile (default: 
	                                        {ProfileStrategies.BY_ZERO_PORT})
	        interval {int} -- interval of time in minutes to build the profile
	                          (default: {30})
	        params {dict} -- params to send to the service (default: {None})
		
	    Returns:
	        dict -- dictionary with the sensor's profile
		"""
		return add_humanize_method(profile.sensor_profile(
			self.portia_config,
			self.edge_id,
			self.port,
			self.sensor,
			strategy,
			interval,
			params
		))

	def select(self, last: bool=False, params: dict=None) -> object:
		"""Retrieves a device's series by its port and sensor.

		Keyword Arguments:
			last {bool} -- if the last package should be returned or not
						   (default: {False})
			params {dict} -- params to send to the service (default: {None})

		Returns:
			object -- object with the device's dimensions
		"""
		return add_humanize_method(select.query_by_port_sensor(
			self.portia_config,
			self.edge_id,
			self.port,
			self.sensor,
			last,
			params
		))

	def summary(
		self,
		strategy: 'SummaryStrategies'=summary.SummaryStrategies.PER_HOUR,
		interval=1,
		params=None
	) -> object:
		"""Summarizes a device by port and sensor.

		Keyword Arguments:
			strategy {SummaryStrategies} -- strategy to use when summarizing
											(default:
											{SummaryStrategies.PER_HOUR})
			interval {int} -- interval of time to summarize (default: {1})
			params {dict} -- params to send to the service (default: {None})

		Returns:
			object -- object with the device's summarized dimensions
		"""
		return add_humanize_method(summary.query_by_port_sensor(
			self.portia_config,
			self.edge_id,
			self.port,
			self.sensor,
			strategy,
			interval,
			params
		))

	def events(self, last: bool=False, params: dict=None) -> object:
		"""Retrieves a device's events by its port and sensor.

		Keyword Arguments:
			last {bool} -- if the last event should be returned or not
						   (default: {False})
			params {dict} -- params to send to the service (default: {None})

		Returns:
			object -- object with the device's events
		"""
		return add_humanize_method(events.query_by_port_sensor(
			self.portia_config,
			self.edge_id,
			self.port,
			self.sensor,
			last,
			params
		))


class EdgeDeviceDimensionFromDevice(object):
	"""Abstracts usage of all Portia endpoints concerning data that only need
	an Edge ID and a dimension code.
	"""
	def __init__(
		self,
		edge_device: EdgeDevice,
		dimension: int,
		portia_config: dict
	):
		"""EdgeDeviceDimensionFromDevice's constructor.
		
		Arguments:
			edge_device {EdgeDevice} -- instance of an Edge device
			dimension {int} -- dimension code of the device
			portia_config {dict} -- Portia's configuration arguments
		"""
		self.edge_id = edge_device.edge_id
		self.dimension = dimension
		self.portia_config = portia_config

	def summary(
		self,
		series: list=None,
		strategy: 'SummaryStrategies'=summary.SummaryStrategies.PER_HOUR,
		interval=1,
		params=None
	) -> object:
		"""Summarizes a device by dimension code.

		Keyword Arguments:
			series {list} -- list of series to summarize (default: {None})
			strategy {SummaryStrategies} -- strategy to use when summarizing
											(default:
											{SummaryStrategies.PER_HOUR})
			interval {int} -- interval of time to summarize (default: {1})
			params {dict} -- params to send to the service (default: {None})

		Returns:
			object -- object with the device's summarized dimensions
		"""
		return add_humanize_method(summary.query_device_by_dimension(
			self.portia_config,
			self.edge_id,
			self.dimension,
			series,
			strategy,
			interval,
			params
		))


class EdgeDeviceDimensionFromPort(object):
	"""Abstracts usage of all Portia endpoints concerning data that only need
	an Edge ID, a port and a dimension code.
	"""
	def __init__(
		self,
		edge_device_port: EdgeDevicePort,
		dimension: int,
		portia_config: dict
	):
		"""EdgeDeviceDimensionFromPort's constructor.
		
		Arguments:
			edge_device_port {EdgeDevicePort} -- instance of an Edge device
												 port
			dimension {int} -- dimension code of the device
			portia_config {dict} -- Portia's configuration arguments
		"""
		self.edge_id = edge_device_port.edge_id
		self.port = edge_device_port.port
		self.dimension = dimension
		self.portia_config = portia_config

	def event(self, event: int) -> 'EdgeDeviceEventFromDimension':
		"""Builds a new EdgeDeviceEventFromDimension instance.
		
		Arguments:
			event {int} -- event code of the device

		Returns:
			EdgeDeviceEventFromDimension -- EdgeDeviceEventFromDimension
											instance
		"""
		return EdgeDeviceEventFromDimension(self, event, self.portia_config)

	def select(self, last: bool=False, params: dict=None) -> object:
		"""Retrieves a device's series by its port and dimension code.

		Keyword Arguments:
			last {bool} -- if the last package should be returned or not
						   (default: {False})
			params {dict} -- params to send to the service (default: {None})

		Returns:
			object -- object with the device's dimensions
		"""
		return add_humanize_method(select.query_by_port_dimension(
			self.portia_config,
			self.edge_id,
			self.port,
			self.dimension,
			last,
			params
		))

	def events(self, last: bool=False, params: dict=None) -> object:
		"""Retrieves a device's events by its port and dimension code.

		Keyword Arguments:
			last {bool} -- if the last event should be returned or not
						   (default: {False})
			params {dict} -- params to send to the service (default: {None})

		Returns:
			object -- object with the device's events
		"""
		return add_humanize_method(events.query_by_port_dimension(
			self.portia_config,
			self.edge_id,
			self.port,
			self.dimension,
			last,
			params
		))


class EdgeDeviceDimensionFromSensor(object):
	"""Abstracts usage of all Portia endpoints concerning data that only need
	an Edge ID, a port, a sensor and a dimension code.
	"""
	def __init__(
		self,
		edge_device_sensor: EdgeDeviceSensor,
		dimension: int,
		portia_config: dict
	):
		"""EdgeDeviceDimensionFromSensor's constructor.
		
		Arguments:
			edge_device_sensor {EdgeDeviceSensor} -- instance of an Edge device
					 								 sensor
			dimension {int} -- dimension code of the device
			portia_config {dict} -- Portia's configuration arguments
		"""
		self.edge_id = edge_device_sensor.edge_id
		self.port = edge_device_sensor.port
		self.sensor = edge_device_sensor.sensor
		self.dimension = dimension
		self.portia_config = portia_config

	def event(self, event: int) -> 'EdgeDeviceEventFromSensorDimension':
		"""Builds a new EdgeDeviceEventFromSensorDimension instance.
		
		Arguments:
			event {int} -- event code of the device

		Returns:
			EdgeDeviceEventFromSensorDimension -- 
				EdgeDeviceEventFromSensorDimension instance
		"""
		return EdgeDeviceEventFromSensorDimension(
			self, event, self.portia_config
		)

	def select(self, last: bool=False, params: dict=None) -> object:
		"""Retrieves a device's series by its port, sensor and dimension code.

		Keyword Arguments:
			last {bool} -- if the last package should be returned or not
						   (default: {False})
			params {dict} -- params to send to the service (default: {None})

		Returns:
			object -- object with the device's dimensions
		"""
		return add_humanize_method(select.query_by_port_sensor_dimension(
			self.portia_config,
			self.edge_id,
			self.port,
			self.sensor,
			self.dimension,
			last,
			params
		))

	def summary(
		self,
		strategy: 'SummaryStrategies'=summary.SummaryStrategies.PER_HOUR,
		interval=1,
		params=None
	) -> object:
		"""Summarizes a device by port, sensor and dimension code.

		Keyword Arguments:
			strategy {SummaryStrategies} -- strategy to use when summarizing
											(default:
											{SummaryStrategies.PER_HOUR})
			interval {int} -- interval of time to summarize (default: {1})
			params {dict} -- params to send to the service (default: {None})

		Returns:
			object -- object with the device's summarized dimensions
		"""
		return add_humanize_method(summary.query_by_port_sensor_dimension(
			self.portia_config,
			self.edge_id,
			self.port,
			self.sensor,
			self.dimension,
			strategy,
			interval,
			params
		))

	def events(self, last: bool=False, params: dict=None) -> object:
		"""Retrieves a device's events by its port, sensor and dimension code.

		Keyword Arguments:
			last {bool} -- if the last event should be returned or not
						   (default: {False})
			params {dict} -- params to send to the service (default: {None})

		Returns:
			object -- object with the device's events
		"""
		return add_humanize_method(events.query_by_port_sensor_dimension(
			self.portia_config,
			self.edge_id,
			self.port,
			self.sensor,
			self.dimension,
			last,
			params
		))


class EdgeDeviceEventFromSensor(object):
	"""Abstracts usage of all Portia endpoints concerning data that only need
	an Edge ID, a port a sensor and an event code.
	"""
	def __init__(
		self,
		edge_device_sensor: EdgeDeviceSensor,
		event: int,
		portia_config: dict
	):
		"""EdgeDeviceEventFromSensor's constructor.
		
		Arguments:
			edge_device_sensor {EdgeDeviceSensor} -- instance of an Edge device
												 	 sensor
			event {int} -- event code of the device
			portia_config {dict} -- Portia's configuration arguments
		"""
		self.edge_id = edge_device_sensor.edge_id
		self.port = edge_device_sensor.port
		self.sensor = edge_device_sensor.sensor
		self.event = event
		self.portia_config = portia_config

	def events(self, last: bool=False, params: dict=None) -> object:
		"""Retrieves a device's events by its port, sensor and event code.

		Keyword Arguments:
			last {bool} -- if the last event should be returned or not
						   (default: {False})
			params {dict} -- params to send to the service (default: {None})

		Returns:
			object -- object with the device's events
		"""
		return add_humanize_method(events.query_by_port_sensor_event(
			self.portia_config,
			self.edge_id,
			self.port,
			self.sensor,
			self.event,
			last,
			params
		))


class EdgeDeviceEventFromDimension(object):
	"""Abstracts usage of all Portia endpoints concerning data that only need
	an Edge ID, a port a dimension code and an event code.
	"""
	def __init__(
		self,
		edge_device_dimension_from_port: EdgeDeviceDimensionFromPort,
		event: int,
		portia_config: dict
	):
		"""EdgeDeviceEventFromDimension's constructor.
		
		Arguments:
			edge_device_dimension_from_port {EdgeDeviceDimensionFromPort} -- 
				instance of an Edge device dimension from port
			event {int} -- event code of the device
			portia_config {dict} -- Portia's configuration arguments
		"""
		self.edge_id = edge_device_dimension_from_port.edge_id
		self.port = edge_device_dimension_from_port.port
		self.dimension = edge_device_dimension_from_port.dimension
		self.event = event
		self.portia_config = portia_config

	def events(self, last: bool=False, params: dict=None) -> object:
		"""Retrieves a device's events by its port, dimension code and event
		code.

		Keyword Arguments:
			last {bool} -- if the last event should be returned or not
						   (default: {False})
			params {dict} -- params to send to the service (default: {None})

		Returns:
			object -- object with the device's events
		"""
		return add_humanize_method(events.query_by_port_dimension_event(
			self.portia_config,
			self.edge_id,
			self.port,
			self.dimension,
			self.event,
			last,
			params
		))


class EdgeDeviceEventFromSensorDimension(object):
	"""Abstracts usage of all Portia endpoints concerning data that only need
	an Edge ID, a port, sensor, dimension code and an event code.
	"""
	def __init__(
		self,
		edge_device_dimension_from_sensor: EdgeDeviceDimensionFromSensor,
		event: int,
		portia_config: dict
	):
		"""EdgeDeviceEventFromDimension's constructor.
		
		Arguments:
			edge_device_dimension_from_sensor {EdgeDeviceDimensionFromSensor}
				-- instance of an Edge device dimension from sensor
			event {int} -- event code of the device
			portia_config {dict} -- Portia's configuration arguments
		"""
		self.edge_id = edge_device_dimension_from_sensor.edge_id
		self.port = edge_device_dimension_from_sensor.port
		self.sensor = edge_device_dimension_from_sensor.sensor
		self.dimension = edge_device_dimension_from_sensor.dimension
		self.event = event
		self.portia_config = portia_config

	def events(self, last: bool=False, params: dict=None) -> object:
		"""Retrieves a device's events by its port, sensor, dimension code and
		event code.

		Keyword Arguments:
			last {bool} -- if the last event should be returned or not
						   (default: {False})
			params {dict} -- params to send to the service (default: {None})

		Returns:
			object -- object with the device's events
		"""
		return add_humanize_method(events.query_by_port_sensor_dimension_event(
			self.portia_config,
			self.edge_id,
			self.port,
			self.sensor,
			self.dimension,
			self.event,
			last,
			params
		))



#####################################
#           Specifications          #
#####################################

# Libraries
import json                       # JSON encoder and decoder
import portiapy.utils as utils

# Functions
def about():
    print("portiapy.specs - an Agriness Edge project")

########################################
# index
########################################
# /pipeline/specs
########################################
def index(portiaConfig):

	endpoint = '/pipeline/specs'
	response = utils.httpGetRequest(portiaConfig, endpoint)

	if response.status_code == 200:

		d = json.loads(response.text)
		if portiaConfig['debug']:
			print( '[portia-debug]: {0}'.format(d) )

		return d

	else:

		d = json.loads(response.text)
		raise Exception( 'couldn\'t list specifications: {0}'.format(d['message']) )

########################################
# store
########################################
# /pipeline/specs
########################################
def store(portiaConfig, payload):

	endpoint = '/pipeline/specs'
	response = utils.httpPostRequest(portiaConfig, endpoint, payload)

	if response.status_code == 200:

		d = json.loads(response.text)
		if portiaConfig['debug']:
			print( '[portia-debug]: {0}'.format(d) )

		return d

	else:

		d = json.loads(response.text)
		raise Exception( 'couldn\'t create specification: {0}'.format(d['message']) )

########################################
# show
########################################
# /pipeline/specs/:spec
########################################
def show(portiaConfig, specName):

	endpoint = '/pipeline/specs/{0}'.format(specName)
	response = utils.httpGetRequest(portiaConfig, endpoint)

	if response.status_code == 200:

		d = json.loads(response.text)
		if portiaConfig['debug']:
			print( '[portia-debug]: {0}'.format(d) )

		return d

	else:

		d = json.loads(response.text)
		raise Exception( 'couldn\'t show specification: {0}'.format(d['message']) )

########################################
# update
########################################
# /pipeline/specs/:spec
########################################
def update(portiaConfig, specName, payload):

	endpoint = '/pipeline/specs/{0}'.format(specName)
	response = utils.httpPutRequest(portiaConfig, endpoint, payload)

	if response.status_code == 200:

		d = json.loads(response.text)
		if portiaConfig['debug']:
			print( '[portia-debug]: {0}'.format(d) )

		return d

	else:

		d = json.loads(response.text)
		raise Exception( 'couldn\'t update specification: {0}'.format(d['message']) )

########################################
# destroy
########################################
# /pipeline/specs/:spec
########################################
def destroy(portiaConfig, specName):

	endpoint = '/pipeline/specs/{0}'.format(specName)
	response = utils.httpDeleteRequest(portiaConfig, endpoint)

	if response.status_code == 200:

		d = json.loads(response.text)
		if portiaConfig['debug']:
			print( '[portia-debug]: {0}'.format(d) )

		return d

	else:

		d = json.loads(response.text)
		raise Exception( 'couldn\'t destroy specification: {0}'.format(d['message']) )
